@@ -3,12 +3,11 @@
 set -e
 
 VERSION=$1
-USER=$2
-PGP_KEYID=$3
+PGP_KEYID=$2
 
 if [ "x$PGP_KEYID" = "x" ]; then
-  echo "try with $0 VERSION GOOGLECODE_USER PGP_KEYID"
-  echo "example: $0 3.0.4 klas@yubico.com 9D1C0E79"
+  echo "try with $0 VERSION PGP_KEYID"
+  echo "example: $0 3.0.4 9D1C0E79"
   exit
 fi
 
@@ -17,6 +16,16 @@ if ! head -3 NEWS  | grep -q "Version $VERSION .released `date -I`"; then
   exit
 fi
 
+if ! head -5 YKPersonalization.pro | grep -q "VERSION.*= \"$VERSION\""; then
+  echo "You need to update version in YKPersonalization.pro"
+  exit
+fi
+
+if [ "x$YUBICO_GITHUB_REPO" = "x" ]; then
+  echo "you need to define YUBICO_GITHUB_REPO"
+  exit
+fi
+
 releasename=yubikey-personalization-gui-${VERSION}
 
 git push
@@ -29,6 +38,6 @@ git archive $releasename --format=tar | tar -xC $releasedir
 git2cl > $releasedir/ChangeLog
 tar -cz --directory=$tmpdir --file=${releasename}.tar.gz $releasename
 gpg --detach-sign --default-key $PGP_KEYID ${releasename}.tar.gz
-googlecode_upload.py -s "OpenPGP signature for ${releasename}." -p yubikey-personalization -u $USER ${releasename}.tar.gz.sig
-googlecode_upload.py -s "$releasename" -p yubikey-personalization -u $USER ${releasename}.tar.gz
-rm -rf $releasedir
+$YUBICO_GITHUB_REPO/publish yubikey-personalization-gui $VERSION ${releasename}.tar.gz*
+rm -rf $tmpdir
+rm -rf $stagedir
