@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2011-2012 Yubico AB.  All rights reserved.
+Copyright (C) 2011-2013 Yubico AB.  All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
@@ -30,8 +30,9 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "ui_chalresppage.h"
 #include "ui/helpbox.h"
 #include "ui/confirmbox.h"
-
-#include <QDesktopServices>
+#include "yubikeywriter.h"
+#include "yubikeyfinder.h"
+#include "yubikeyutil.h"
 #include "common.h"
 
 ChalRespPage::ChalRespPage(QWidget *parent) :
@@ -41,6 +42,7 @@ ChalRespPage::ChalRespPage(QWidget *parent) :
     ui->setupUi(this);
 
     m_ykConfig = 0;
+    m_keyPresent = false;
     clearState();
 
     //Connect pages
@@ -58,8 +60,20 @@ ChalRespPage::ChalRespPage(QWidget *parent) :
     connect(ui->advResetBtn, SIGNAL(clicked()),
             this, SLOT(resetAdvPage()));
 
+    connect(ui->advRequireUserInputCheck, SIGNAL(clicked(bool)),
+            this, SLOT(requireInput_clicked(bool)));
+    connect(ui->quickRequireUserInputCheck, SIGNAL(clicked(bool)),
+            this, SLOT(requireInput_clicked(bool)));
+
+    connect(ui->advHmacVarInputRadio, SIGNAL(clicked()),
+            this, SLOT(hmac_lt64_clicked()));
+    connect(ui->advHmacFixedInputRadio, SIGNAL(clicked()),
+            this, SLOT(hmac_lt64_clicked()));
+
     ui->quickResultsWidget->resizeColumnsToContents();
     ui->advResultsWidget->resizeColumnsToContents();
+
+    loadSettings();
 }
 
 ChalRespPage::~ChalRespPage() {
@@ -70,6 +84,24 @@ ChalRespPage::~ChalRespPage() {
     delete ui;
 }
 
+void ChalRespPage::loadSettings() {
+    QSettings settings;
+
+    ui->advExportConfigBtn->setVisible(settings.value(SG_EXPORT_PREFERENCE).toBool());
+    ui->quickExportConfigBtn->setVisible(settings.value(SG_EXPORT_PREFERENCE).toBool());
+
+    bool reqInput = settings.value(SG_REQUIRE_INPUT).toBool();
+    ui->quickRequireUserInputCheck->setChecked(reqInput);
+    ui->advRequireUserInputCheck->setChecked(reqInput);
+
+    bool hmaclt64 = true;
+    if(settings.contains(SG_HMAC_LT64)) {
+      hmaclt64 = settings.value(SG_HMAC_LT64).toBool();
+    }
+    ui->advHmacVarInputRadio->setChecked(hmaclt64);
+    ui->advHmacFixedInputRadio->setChecked(!hmaclt64);
+}
+
 /*
  Common
 */
@@ -139,13 +171,11 @@ void ChalRespPage::connectHelpButtons() {
 
     connect(ui->quickConfigHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
     connect(ui->quickParamGenSchemeHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
-    connect(ui->quickConfigProtectionHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
     connect(ui->quickChalRespOptionsHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
     connect(ui->quickPvtIdHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
     connect(ui->quickSecretKeyHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
 
     connect(ui->advConfigHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
-    connect(ui->advConfigProtectionHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
     connect(ui->advChalRespOptionsHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
     connect(ui->advSecretKeyHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
 
@@ -155,13 +185,11 @@ void ChalRespPage::connectHelpButtons() {
 
     mapper->setMapping(ui->quickConfigHelpBtn, HelpBox::Help_ConfigurationSlot);
     mapper->setMapping(ui->quickParamGenSchemeHelpBtn, HelpBox::Help_ParameterGeneration);
-    mapper->setMapping(ui->quickConfigProtectionHelpBtn, HelpBox::Help_ConfigurationProtection);
     mapper->setMapping(ui->quickChalRespOptionsHelpBtn, HelpBox::Help_ChalRespOption);
     mapper->setMapping(ui->quickPvtIdHelpBtn, HelpBox::Help_PrivateID);
     mapper->setMapping(ui->quickSecretKeyHelpBtn, HelpBox::Help_SecretKey);
 
     mapper->setMapping(ui->advConfigHelpBtn, HelpBox::Help_ConfigurationSlot);
-    mapper->setMapping(ui->advConfigProtectionHelpBtn, HelpBox::Help_ConfigurationProtection);
     mapper->setMapping(ui->advChalRespOptionsHelpBtn, HelpBox::Help_ChalRespOption);
     mapper->setMapping(ui->advSecretKeyHelpBtn, HelpBox::Help_SecretKey);
 
@@ -179,7 +207,9 @@ void ChalRespPage::keyFound(bool found, bool* featuresMatrix) {
     if(found) {
         if(m_state == State_Initial) {
             ui->quickWriteConfigBtn->setEnabled(true);
+            ui->quickExportConfigBtn->setEnabled(true);
             ui->advWriteConfigBtn->setEnabled(true);
+            ui->advExportConfigBtn->setEnabled(true);
 
             if(!featuresMatrix[YubiKeyFinder::Feature_MultipleConfigurations]) {
                 ui->quickConfigSlot2Radio->setEnabled(false);
@@ -188,29 +218,42 @@ void ChalRespPage::keyFound(bool found, bool* featuresMatrix) {
                 ui->quickConfigSlot2Radio->setEnabled(true);
                 ui->advConfigSlot2Radio->setEnabled(true);
             }
+            if(!featuresMatrix[YubiKeyFinder::Feature_ChallengeResponseFixed]) {
+                ui->advHmacFixedInputRadio->setEnabled(false);
+                ui->advHmacVarInputRadio->setChecked(true);
+            } else {
+                ui->advHmacFixedInputRadio->setEnabled(true);
+            }
 
             if(!featuresMatrix[YubiKeyFinder::Feature_ChallengeResponse]) {
                 this->setEnabled(false);
             }
-        } else if(m_state >= State_Programming_Multiple) {
+        } else if(m_state >= State_Programming_Multiple && m_keyPresent == false) {
             if(this->currentIndex() == Page_Quick) {
                 if(m_state == State_Programming_Multiple) {
                     ui->quickWriteConfigBtn->setEnabled(true);
+                    ui->quickExportConfigBtn->setEnabled(true);
                 } else {
-                    writeQuickConfig();
+                    writeQuickConfig(WRITE_CONFIG);
                 }
 
             } else {
                 if(m_state == State_Programming_Multiple) {
                     ui->advWriteConfigBtn->setEnabled(true);
+                    ui->advExportConfigBtn->setEnabled(true);
                 } else {
-                    writeAdvConfig();
+                    writeAdvConfig(WRITE_CONFIG);
                 }
             }
         }
+        m_keyPresent = true;
     } else {
         ui->quickWriteConfigBtn->setEnabled(false);
+        ui->quickExportConfigBtn->setEnabled(false);
         ui->advWriteConfigBtn->setEnabled(false);
+        ui->advExportConfigBtn->setEnabled(false);
+        ui->advHmacFixedInputRadio->setEnabled(true);
+        m_keyPresent = false;
 
         if(m_state == State_Initial) {
             ui->quickConfigSlot2Radio->setEnabled(true);
@@ -253,9 +296,7 @@ void ChalRespPage::resetQuickPage() {
     ui->quickAutoProgramKeysCheck->setChecked(false);
     ui->quickProgramMulKeysBox->setChecked(false);
 
-    ui->quickConfigProtectionCombo->setCurrentIndex(0);
-
-    ui->quickRequireUserInputCheck->setChecked(false);
+    ui->quickConfigProtectionBox->reset();
 
     ui->quickPvtIdCheck->setChecked(true);
     ui->quickPvtIdTxt->clear();
@@ -276,6 +317,7 @@ void ChalRespPage::freezeQuickPage(bool freeze) {
     ui->quickKeyParamsBox->setEnabled(disable);
 
     ui->quickWriteConfigBtn->setEnabled(disable);
+    ui->quickExportConfigBtn->setEnabled(disable);
     ui->quickStopBtn->setEnabled(!disable);
     ui->quickResetBtn->setEnabled(disable);
     ui->quickBackBtn->setEnabled(disable);
@@ -287,56 +329,10 @@ void ChalRespPage::on_quickProgramMulKeysBox_clicked(bool checked) {
     }
 }
 
-void ChalRespPage::on_quickConfigParamsCombo_currentIndexChanged(int index) {
+void ChalRespPage::on_quickConfigParamsCombo_currentIndexChanged(__attribute__((unused)) int index) {
     changeQuickConfigParams();
 }
 
-void ChalRespPage::on_quickConfigProtectionCombo_currentIndexChanged(int index) {
-    switch(index) {
-    case CONFIG_PROTECTION_DISABLED:
-        ui->quickCurrentAccessCodeTxt->clear();
-        ui->quickCurrentAccessCodeTxt->setEnabled(false);
-
-        ui->quickNewAccessCodeTxt->clear();
-        ui->quickNewAccessCodeTxt->setEnabled(false);
-        break;
-    case CONFIG_PROTECTION_ENABLE:
-        ui->quickCurrentAccessCodeTxt->clear();
-        ui->quickCurrentAccessCodeTxt->setEnabled(false);
-
-        on_quickNewAccessCodeTxt_editingFinished();
-        ui->quickNewAccessCodeTxt->setEnabled(true);
-        break;
-    case CONFIG_PROTECTION_DISABLE:
-    case CONFIG_PROTECTION_ENABLED:
-        on_quickCurrentAccessCodeTxt_editingFinished();
-        ui->quickCurrentAccessCodeTxt->setEnabled(true);
-
-        ui->quickNewAccessCodeTxt->clear();
-        ui->quickNewAccessCodeTxt->setEnabled(false);
-        break;
-    case CONFIG_PROTECTION_CHANGE:
-        on_quickCurrentAccessCodeTxt_editingFinished();
-        ui->quickCurrentAccessCodeTxt->setEnabled(true);
-
-        on_quickNewAccessCodeTxt_editingFinished();
-        ui->quickNewAccessCodeTxt->setEnabled(true);
-        break;
-    }
-}
-
-void ChalRespPage::on_quickCurrentAccessCodeTxt_editingFinished() {
-    QString txt = ui->quickCurrentAccessCodeTxt->text();
-    YubiKeyUtil::qstrClean(&txt, (size_t)ACC_CODE_SIZE * 2);
-    ui->quickCurrentAccessCodeTxt->setText(txt);
-}
-
-void ChalRespPage::on_quickNewAccessCodeTxt_editingFinished() {
-    QString txt = ui->quickNewAccessCodeTxt->text();
-    YubiKeyUtil::qstrClean(&txt, (size_t)ACC_CODE_SIZE * 2);
-    ui->quickNewAccessCodeTxt->setText(txt);
-}
-
 void ChalRespPage::on_quickPvtIdCheck_stateChanged(int state) {
     if(ui->quickConfigParamsCombo->currentIndex() != SCHEME_RAND_ALL) {
         bool disable = (state != 0);
@@ -396,9 +392,23 @@ void ChalRespPage::on_quickWriteConfigBtn_clicked() {
         m_state = State_Programming;
     }
 
-    writeQuickConfig();
+    writeQuickConfig(WRITE_CONFIG);
 }
 
+void ChalRespPage::on_quickExportConfigBtn_clicked() {
+    emit showStatusMessage(NULL, -1);
+
+    //Validate settings
+    if(!validateQuickSettings()) {
+        return;
+    }
+
+    clearState();
+
+    freezeQuickPage(true);
+
+    writeQuickConfig(EXPORT_CONFIG);
+}
 void ChalRespPage::on_quickStopBtn_clicked() {
     ui->quickStopBtn->setEnabled(false);
     m_state = State_Initial;
@@ -406,7 +416,7 @@ void ChalRespPage::on_quickStopBtn_clicked() {
 }
 
 void ChalRespPage::stopQuickConfigWritting() {
-    qDebug() << "Stopping quick confgiuration writing...";
+    qDebug() << "Stopping quick configuration writing...";
 
     if(m_state >= State_Programming_Multiple) {
         ui->quickStopBtn->setEnabled(true);
@@ -483,27 +493,14 @@ bool ChalRespPage::validateQuickSettings() {
         }
     }
 
-    //Check if logging is disabled and
-    //configuration protection is being enabled
-    if(!settings.value(SG_ENABLE_CONF_PROTECTION).toBool() &&
-       !YubiKeyLogger::isLogging() &&
-       ui->quickConfigProtectionCombo->currentIndex() == CONFIG_PROTECTION_ENABLE) {
-        //Confirm from client
-        ConfirmBox confirm(this);
-        confirm.setConfirmIndex(ConfirmBox::Confirm_ConfigurationProtection);
-        int ret = confirm.exec();
-
-        switch (ret) {
-        case 1:     //Yes
-            break;
-        default:    //No
-            return false;
-        }
+    if(!ui->quickConfigProtectionBox->checkConfirm()) {
+        return false;
     }
+
     return true;
 }
 
-void ChalRespPage::writeQuickConfig() {
+void ChalRespPage::writeQuickConfig(int mode) {
     qDebug() << "Writing configuration...";
 
     //Disable stop button while configuration is being written
@@ -520,6 +517,9 @@ void ChalRespPage::writeQuickConfig() {
     //Programming mode...
     m_ykConfig->setProgrammingMode(YubiKeyConfig::Mode_ChalRespYubico);
 
+    // set serial
+    m_ykConfig->setSerial(QString::number(YubiKeyFinder::getInstance()->serial()));
+
     //Configuration slot...
     int configSlot = 1;
     if(ui->quickConfigSlot2Radio->isChecked()) {
@@ -537,25 +537,29 @@ void ChalRespPage::writeQuickConfig() {
 
     //Configuration protection...
     //Current Access Code...
-    m_ykConfig->setCurrentAccessCodeTxt(ui->quickCurrentAccessCodeTxt->text());
-
-    //New Access Code...
-    if(ui->quickConfigProtectionCombo->currentIndex()
-        == CONFIG_PROTECTION_DISABLE){
-        m_ykConfig->setNewAccessCodeTxt(ACCESS_CODE_DEFAULT);
-    } else {
-        m_ykConfig->setNewAccessCodeTxt(ui->quickNewAccessCodeTxt->text());
-    }
+    m_ykConfig->setCurrentAccessCodeTxt(
+            ui->quickConfigProtectionBox->currentAccessCode());
+    m_ykConfig->setNewAccessCodeTxt(
+            ui->quickConfigProtectionBox->newAccessCode(),
+            ui->quickConfigProtectionBox->newAccMode());
 
     //Challenge-Response Options...
     m_ykConfig->setChalYubico(true);
     m_ykConfig->setChalBtnTrig(ui->quickRequireUserInputCheck->isChecked());
 
-    //Write
-    connect(YubiKeyWriter::getInstance(), SIGNAL(configWritten(bool, const QString &)),
-            this, SLOT(quickConfigWritten(bool, const QString &)));
+    if(mode == WRITE_CONFIG) {
+        //Write
+        connect(YubiKeyWriter::getInstance(), SIGNAL(configWritten(bool, const QString &)),
+                this, SLOT(quickConfigWritten(bool, const QString &)));
 
-    YubiKeyWriter::getInstance()->writeConfig(m_ykConfig);
+        YubiKeyWriter::getInstance()->writeConfig(m_ykConfig);
+    } else if(mode == EXPORT_CONFIG) {
+        //Write
+        connect(YubiKeyWriter::getInstance(), SIGNAL(configWritten(bool, const QString &)),
+                this, SLOT(quickConfigExported(bool, const QString &)));
+
+        YubiKeyWriter::getInstance()->exportConfig(m_ykConfig);
+    }
 }
 
 void ChalRespPage::quickConfigWritten(bool written, const QString &msg) {
@@ -590,6 +594,28 @@ void ChalRespPage::quickConfigWritten(bool written, const QString &msg) {
     stopQuickConfigWritting();
 }
 
+void ChalRespPage::quickConfigExported(bool written, const QString &msg) {
+    disconnect(YubiKeyWriter::getInstance(), SIGNAL(configWritten(bool, const QString &)),
+               this, SLOT(quickConfigExported(bool, const QString &)));
+
+    QString message;
+
+    if(written) {
+        qDebug() << "Configuration exported....";
+
+        message = KEY_EXPORTED;
+
+        showStatusMessage(message, 0);
+    } else {
+        qDebug() << "Configuration could not be exported....";
+
+        message = msg;
+    }
+
+    m_ready = false;
+    stopQuickConfigWritting();
+}
+
 void ChalRespPage::quickUpdateResults(bool written, const QString &msg) {
     int row = 0;
 
@@ -639,10 +665,7 @@ void ChalRespPage::resetAdvPage() {
     ui->advAutoProgramKeysCheck->setChecked(false);
     ui->advProgramMulKeysBox->setChecked(false);
 
-    ui->advConfigProtectionCombo->setCurrentIndex(0);
-
-    ui->advRequireUserInputCheck->setChecked(false);
-    ui->advHmacVarInputRadio->setChecked(true);
+    ui->advConfigProtectionBox->reset();
 
     ui->advSecretKeyTxt->clear();
     on_advSecretKeyTxt_editingFinished();
@@ -659,6 +682,7 @@ void ChalRespPage::freezeAdvPage(bool freeze) {
     ui->advKeyParamsBox->setEnabled(disable);
 
     ui->advWriteConfigBtn->setEnabled(disable);
+    ui->advExportConfigBtn->setEnabled(disable);
     ui->advStopBtn->setEnabled(!disable);
     ui->advResetBtn->setEnabled(disable);
     ui->advBackBtn->setEnabled(disable);
@@ -670,56 +694,10 @@ void ChalRespPage::on_advProgramMulKeysBox_clicked(bool checked) {
     }
 }
 
-void ChalRespPage::on_advConfigParamsCombo_currentIndexChanged(int index) {
+void ChalRespPage::on_advConfigParamsCombo_currentIndexChanged(__attribute__((unused)) int index) {
     changeAdvConfigParams();
 }
 
-void ChalRespPage::on_advConfigProtectionCombo_currentIndexChanged(int index) {
-    switch(index) {
-    case CONFIG_PROTECTION_DISABLED:
-        ui->advCurrentAccessCodeTxt->clear();
-        ui->advCurrentAccessCodeTxt->setEnabled(false);
-
-        ui->advNewAccessCodeTxt->clear();
-        ui->advNewAccessCodeTxt->setEnabled(false);
-        break;
-    case CONFIG_PROTECTION_ENABLE:
-        ui->advCurrentAccessCodeTxt->clear();
-        ui->advCurrentAccessCodeTxt->setEnabled(false);
-
-        on_advNewAccessCodeTxt_editingFinished();
-        ui->advNewAccessCodeTxt->setEnabled(true);
-        break;
-    case CONFIG_PROTECTION_DISABLE:
-    case CONFIG_PROTECTION_ENABLED:
-        on_advCurrentAccessCodeTxt_editingFinished();
-        ui->advCurrentAccessCodeTxt->setEnabled(true);
-
-        ui->advNewAccessCodeTxt->clear();
-        ui->advNewAccessCodeTxt->setEnabled(false);
-        break;
-    case CONFIG_PROTECTION_CHANGE:
-        on_advCurrentAccessCodeTxt_editingFinished();
-        ui->advCurrentAccessCodeTxt->setEnabled(true);
-
-        on_advNewAccessCodeTxt_editingFinished();
-        ui->advNewAccessCodeTxt->setEnabled(true);
-        break;
-    }
-}
-
-void ChalRespPage::on_advCurrentAccessCodeTxt_editingFinished() {
-    QString txt = ui->advCurrentAccessCodeTxt->text();
-    YubiKeyUtil::qstrClean(&txt, (size_t)ACC_CODE_SIZE * 2);
-    ui->advCurrentAccessCodeTxt->setText(txt);
-}
-
-void ChalRespPage::on_advNewAccessCodeTxt_editingFinished() {
-    QString txt = ui->advNewAccessCodeTxt->text();
-    YubiKeyUtil::qstrClean(&txt, (size_t)ACC_CODE_SIZE * 2);
-    ui->advNewAccessCodeTxt->setText(txt);
-}
-
 void ChalRespPage::on_advSecretKeyTxt_editingFinished() {
     QString txt = ui->advSecretKeyTxt->text();
     YubiKeyUtil::qstrClean(&txt, (size_t)KEY_SIZE_OATH * 2);
@@ -761,7 +739,22 @@ void ChalRespPage::on_advWriteConfigBtn_clicked() {
         m_state = State_Programming;
     }
 
-    writeAdvConfig();
+    writeAdvConfig(WRITE_CONFIG);
+}
+
+void ChalRespPage::on_advExportConfigBtn_clicked() {
+    emit showStatusMessage(NULL, -1);
+
+    //Validate settings
+    if(!validateAdvSettings()) {
+        return;
+    }
+
+    clearState();
+
+    freezeAdvPage(true);
+
+    writeAdvConfig(EXPORT_CONFIG);
 }
 
 void ChalRespPage::on_advStopBtn_clicked() {
@@ -771,7 +764,7 @@ void ChalRespPage::on_advStopBtn_clicked() {
 }
 
 void ChalRespPage::stopAdvConfigWritting() {
-    qDebug() << "Stopping adv confgiuration writing...";
+    qDebug() << "Stopping adv configuration writing...";
 
     if(m_state >= State_Programming_Multiple) {
         ui->advStopBtn->setEnabled(true);
@@ -786,11 +779,15 @@ void ChalRespPage::stopAdvConfigWritting() {
 }
 
 void ChalRespPage::changeAdvConfigParams() {
-    //Secret Key...
-    QString secretKeyTxt = YubiKeyUtil::getNextHex(
+    int index = ui->advConfigParamsCombo->currentIndex();
+    // else we use a static secret.
+    if(index == SCHEME_INCR_ID_RAND_SECRET) {
+        //Secret Key...
+        QString secretKeyTxt = YubiKeyUtil::getNextHex(
             KEY_SIZE_OATH * 2,
             ui->advSecretKeyTxt->text(), GEN_SCHEME_RAND);
-    ui->advSecretKeyTxt->setText(secretKeyTxt);
+        ui->advSecretKeyTxt->setText(secretKeyTxt);
+    }
     on_advSecretKeyTxt_editingFinished();
     m_ready = true;
 }
@@ -821,27 +818,14 @@ bool ChalRespPage::validateAdvSettings() {
         }
     }
 
-    //Check if logging is disabled and
-    //configuration protection is being enabled
-    if(!settings.value(SG_ENABLE_CONF_PROTECTION).toBool() &&
-       !YubiKeyLogger::isLogging() &&
-       ui->advConfigProtectionCombo->currentIndex() == CONFIG_PROTECTION_ENABLE) {
-        //Confirm from client
-        ConfirmBox confirm(this);
-        confirm.setConfirmIndex(ConfirmBox::Confirm_ConfigurationProtection);
-        int ret = confirm.exec();
-
-        switch (ret) {
-        case 1:     //Yes
-            break;
-        default:    //No
-            return false;
-        }
+    if(!ui->advConfigProtectionBox->checkConfirm()) {
+        return false;
     }
+
     return true;
 }
 
-void ChalRespPage::writeAdvConfig() {
+void ChalRespPage::writeAdvConfig(int mode) {
     qDebug() << "Writing configuration...";
 
     //Disable stop button while configuration is being written
@@ -858,6 +842,9 @@ void ChalRespPage::writeAdvConfig() {
     //Programming mode...
     m_ykConfig->setProgrammingMode(YubiKeyConfig::Mode_ChalRespHmac);
 
+    // set serial
+    m_ykConfig->setSerial(QString::number(YubiKeyFinder::getInstance()->serial()));
+
     //Configuration slot...
     int configSlot = 1;
     if(ui->advConfigSlot2Radio->isChecked()) {
@@ -869,27 +856,30 @@ void ChalRespPage::writeAdvConfig() {
     m_ykConfig->setSecretKeyTxt(ui->advSecretKeyTxt->text());
 
     //Configuration protection...
-    //Current Access Code...
-    m_ykConfig->setCurrentAccessCodeTxt(ui->advCurrentAccessCodeTxt->text());
-
-    //New Access Code...
-    if(ui->advConfigProtectionCombo->currentIndex()
-        == CONFIG_PROTECTION_DISABLE){
-        m_ykConfig->setNewAccessCodeTxt(ACCESS_CODE_DEFAULT);
-    } else {
-        m_ykConfig->setNewAccessCodeTxt(ui->advNewAccessCodeTxt->text());
-    }
+    m_ykConfig->setCurrentAccessCodeTxt(
+            ui->advConfigProtectionBox->currentAccessCode());
+    m_ykConfig->setNewAccessCodeTxt(
+            ui->advConfigProtectionBox->newAccessCode(),
+            ui->advConfigProtectionBox->newAccMode());
 
     //Challenge-Response Options...
     m_ykConfig->setChalHmac(true);
     m_ykConfig->setHmacLT64(ui->advHmacVarInputRadio->isChecked());
     m_ykConfig->setChalBtnTrig(ui->advRequireUserInputCheck->isChecked());
 
-    //Write
-    connect(YubiKeyWriter::getInstance(), SIGNAL(configWritten(bool, const QString &)),
-            this, SLOT(advConfigWritten(bool, const QString &)));
+    if(mode == WRITE_CONFIG) {
+        //Write
+        connect(YubiKeyWriter::getInstance(), SIGNAL(configWritten(bool, const QString &)),
+                this, SLOT(advConfigWritten(bool, const QString &)));
 
-    YubiKeyWriter::getInstance()->writeConfig(m_ykConfig);
+        YubiKeyWriter::getInstance()->writeConfig(m_ykConfig);
+    } else if(mode == EXPORT_CONFIG) {
+        //Write
+        connect(YubiKeyWriter::getInstance(), SIGNAL(configWritten(bool, const QString &)),
+                this, SLOT(advConfigExported(bool, const QString &)));
+
+        YubiKeyWriter::getInstance()->exportConfig(m_ykConfig);
+    }
 }
 
 void ChalRespPage::advConfigWritten(bool written, const QString &msg) {
@@ -924,6 +914,28 @@ void ChalRespPage::advConfigWritten(bool written, const QString &msg) {
     stopAdvConfigWritting();
 }
 
+void ChalRespPage::advConfigExported(bool written, const QString &msg) {
+    disconnect(YubiKeyWriter::getInstance(), SIGNAL(configWritten(bool, const QString &)),
+               this, SLOT(advConfigExported(bool, const QString &)));
+
+    QString message;
+
+    if(written) {
+        qDebug() << "Configuration written....";
+
+        message = KEY_EXPORTED;
+
+        showStatusMessage(message, 0);
+    } else {
+        qDebug() << "Configuration could not be exported....";
+
+        message = msg;
+    }
+
+    m_ready = false;
+    stopAdvConfigWritting();
+}
+
 void ChalRespPage::advUpdateResults(bool written, const QString &msg) {
     int row = 0;
 
@@ -959,3 +971,29 @@ void ChalRespPage::advUpdateResults(bool written, const QString &msg) {
     ui->advResultsWidget->resizeColumnsToContents();
     ui->advResultsWidget->resizeRowsToContents();
 }
+
+void ChalRespPage::requireInput_clicked(bool checked) {
+    QSettings settings;
+    settings.setValue(SG_REQUIRE_INPUT, checked);
+    if(m_currentPage == Page_Quick) {
+        ui->advRequireUserInputCheck->setChecked(checked);
+    } else {
+        ui->quickRequireUserInputCheck->setChecked(checked);
+    }
+}
+
+void ChalRespPage::hmac_lt64_clicked() {
+    QSettings settings;
+    settings.setValue(SG_HMAC_LT64, ui->advHmacVarInputRadio->isChecked());
+}
+
+
+void ChalRespPage::setCurrentSlot(int slot) {
+    if(m_currentPage == Page_Advanced) {
+        ui->advConfigSlot1Radio->setChecked(slot == 1);
+        ui->advConfigSlot2Radio->setChecked(slot == 2);
+    } else if(m_currentPage == Page_Quick) {
+        ui->quickConfigSlot1Radio->setChecked(slot == 1);
+        ui->quickConfigSlot2Radio->setChecked(slot == 2);
+    }
+}
