@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2011-2012 Yubico AB.  All rights reserved.
+Copyright (C) 2011-2013 Yubico AB.  All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
@@ -28,6 +28,16 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include "ui/mainwindow.h"
 #include "ui/ui_mainwindow.h"
+#include "yubikeyfinder.h"
+#include "yubikeywriter.h"
+#include "yubikeyutil.h"
+#include "ui/otppage.h"
+#include "ui/oathpage.h"
+#include "ui/staticpage.h"
+#include "ui/chalresppage.h"
+#include "ui/toolpage.h"
+#include "ui/settingpage.h"
+#include "ui/aboutpage.h"
 
 #include "common.h"
 #include "version.h"
@@ -75,12 +85,38 @@ MainWindow::MainWindow(QWidget *parent) :
             m_otpPage, SLOT(loadSettings()));
     connect(m_settingPage, SIGNAL(settingsChanged()),
             m_oathPage, SLOT(loadSettings()));
+    connect(m_settingPage, SIGNAL(settingsChanged()),
+            m_chalRespPage, SLOT(loadSettings()));
+    connect(m_settingPage, SIGNAL(settingsChanged()),
+            m_staticPage, SLOT(loadSettings()));
+    connect(m_settingPage, SIGNAL(settingsChanged()),
+            m_toolPage, SLOT(loadSettings()));
+
+    connect(m_toolPage, SIGNAL(reloadSettings()),
+            m_settingPage, SLOT(reloadSettings()));
+
+    connect(m_toolPage, SIGNAL(switchPage(int, int, int)),
+            this, SLOT(setCurrentPage(int, int, int)));
 
     //Intialize settings
     m_settingPage->init();
 
     //Start YubiKey finder
     YubiKeyFinder::getInstance()->start();
+
+    // add action for toggling animation
+    animationAction = new QAction(this);
+    animationAction->setText("Device image animated");
+    animationAction->setCheckable(true);
+    connect(animationAction, SIGNAL(triggered(bool)), this, SLOT(toggleAnimation(bool)));
+    ui->deviceImage->addAction(animationAction);
+
+    QSettings settings;
+    if(settings.value(SG_ANIMATIONS_PREFERENCE, true).toBool()) {
+        animationAction->setChecked(true);
+    } else {
+        animationAction->setChecked(false);
+    }
 }
 
 MainWindow::~MainWindow()
@@ -96,6 +132,8 @@ MainWindow::~MainWindow()
     delete m_toolPage;
     delete m_aboutPage;
 
+    delete animationAction;
+
     delete ui;
 }
 
@@ -156,7 +194,7 @@ void MainWindow::createPages() {
     setCurrentPage(Page_About);
 }
 
-void MainWindow::setCurrentPage(int pageIndex) {
+void MainWindow::setCurrentPage(int pageIndex, int tab, int slot) {
     // Page changed...
 
     // Clear status message
@@ -206,21 +244,31 @@ void MainWindow::setCurrentPage(int pageIndex) {
     switch(pageIndex){
     case Page_Otp:
         ui->otpMenuBtn->setStyleSheet(checkedMenuBtnSS);
+        if(tab) m_otpPage->setCurrentPage(tab);
+        if(slot) m_otpPage->setCurrentSlot(slot);
         break;
     case Page_Oath:
         ui->oathHotpMenuBtn->setStyleSheet(checkedMenuBtnSS);
+        if(tab) m_oathPage->setCurrentPage(tab);
+        if(slot) m_oathPage->setCurrentSlot(slot);
         break;
     case Page_Static:
         ui->staticMenuBtn->setStyleSheet(checkedMenuBtnSS);
+        if(tab) m_staticPage->setCurrentPage(tab);
+        if(slot) m_staticPage->setCurrentSlot(slot);
         break;
     case Page_ChalResp:
         ui->chalRespMenuBtn->setStyleSheet(checkedMenuBtnSS);
+        if(tab) m_chalRespPage->setCurrentPage(tab);
+        if(slot) m_chalRespPage->setCurrentSlot(slot);
         break;
     case Page_Settings:
         ui->settingsMenuBtn->setStyleSheet(checkedMenuBtnSS);
+        if(tab) m_settingPage->setCurrentPage(tab);
         break;
     case Page_Tools:
         ui->toolsMenuBtn->setStyleSheet(checkedMenuBtnSS);
+        if(tab) m_toolPage->setCurrentPage(tab);
         break;
     case Page_About:
         ui->aboutMenuBtn->setStyleSheet(checkedMenuBtnSS);
@@ -260,6 +308,10 @@ void MainWindow::resetDeviceInfo() {
     ui->scanCodeSupportLbl->setText(NA);
     ui->chalRespSupportLbl->setPixmap(NULL);
     ui->chalRespSupportLbl->setText(NA);
+    ui->updatableSupportLbl->setPixmap(NULL);
+    ui->updatableSupportLbl->setText(NA);
+    ui->ndefSupportLbl->setPixmap(NULL);
+    ui->ndefSupportLbl->setText(NA);
 }
 
 void MainWindow::keyFound(bool found, bool* featuresMatrix) {
@@ -271,11 +323,60 @@ void MainWindow::keyFound(bool found, bool* featuresMatrix) {
 
     resetDeviceInfo();
 
+    if(ui->deviceImage->movie()) {
+      delete(ui->deviceImage->movie());
+      ui->deviceImage->setMovie(NULL);
+    }
+
     if(found) {
+        YubiKeyFinder *finder = YubiKeyFinder::getInstance();
+        int touchLevel = finder->touchLevel();
+        qDebug() << "touchLevel is " << touchLevel;
+        if(touchLevel & CONFIG1_VALID && touchLevel & CONFIG2_VALID) {
+            ui->programLbl->setText(tr("Slot 1 and 2 configured"));
+        } else if(touchLevel & CONFIG1_VALID) {
+            ui->programLbl->setText(tr("Slot 1 configured"));
+        } else if(touchLevel & CONFIG2_VALID) {
+            ui->programLbl->setText(tr("Slot 2 configured"));
+        } else {
+            ui->programLbl->setText(tr("Not configured"));
+        }
+
+        unsigned int version = finder->version();
         ui->statusLbl->setText(KEY_FOUND);
         ui->statusLbl->setStyleSheet(QString::fromUtf8(SS_YKSTATUS_SUCCESS));
 
-        ui->versionLbl->setText(YubiKeyFinder::getInstance()->versionStr());
+        ui->versionLbl->setText(finder->versionStr());
+        qDebug() << "version is" << finder->versionStr();
+
+        QPixmap pixmap;
+        QMovie *movie = new QMovie();
+        if(version < YK_VERSION(2,0,0)) {
+            pixmap.load(":/res/images/v1-3-not-animated.png");
+        } else if(version < YK_VERSION(2,1,4)) {
+            movie->setFileName(":/res/images/V2-0-2-1-animated.mng");
+        } else if(version < YK_VERSION(2,2,0)) {
+            // YubiKey NEO
+            pixmap.load(":/res/images/neo_transparent.png");
+        } else if(version % 10 == 9){
+            pixmap.load(":/res/images/yubikey_devel.png");
+        } else if(version < YK_VERSION(2,3,0)){
+            movie->setFileName(":/res/images/v2-2-animated.mng");
+        } else if(version < YK_VERSION(3,0,0)){
+            movie->setFileName(":/res/images/v2-3-animated.mng");
+        } else { // only case left should be version over 3, that is neo
+            pixmap.load(":/res/images/neo_production.png");
+        }
+        if(pixmap.isNull()) {
+            ui->deviceImage->setMovie(movie);
+            movie->start();
+            if(!animationAction->isChecked()) {
+                movie->stop();
+            }
+        } else {
+            delete movie;
+            ui->deviceImage->setPixmap(pixmap);
+        }
 
         unsigned int serial = 0;
         if(featuresMatrix[YubiKeyFinder::Feature_SerialNumber]) {
@@ -370,7 +471,24 @@ void MainWindow::keyFound(bool found, bool* featuresMatrix) {
             ui->chalRespMenuBtn->setStyleSheet(disabledMenuBtnSS);
             ui->chalRespMenuBtn->setEnabled(false);
         }
+
+        if(featuresMatrix[YubiKeyFinder::Feature_Updatable]) {
+            ui->updatableSupportLbl->setPixmap(TICKMAP);
+        } else {
+            ui->updatableSupportLbl->setPixmap(CROSSMAP);
+        }
+
+        if(featuresMatrix[YubiKeyFinder::Feature_Ndef]) {
+            ui->ndefSupportLbl->setPixmap(TICKMAP);
+        } else {
+            ui->ndefSupportLbl->setPixmap(CROSSMAP);
+        }
     } else {
+        ui->programLbl->clear();
+        if(ui->deviceImage->pixmap()) {
+            ui->deviceImage->setPixmap(NULL);
+        }
+        ui->deviceImage->clear();
         ui->statusLbl->setText(NO_KEY_FOUND);
         ui->statusLbl->setStyleSheet(QString::fromUtf8(SS_YKSTATUS_ERROR));
 
@@ -462,3 +580,16 @@ void MainWindow::on_serialNoHexCopyBtn_clicked() {
 void MainWindow::on_serialNoModhexCopyBtn_clicked() {
     copyToClipboard(ui->serialNoModhexLbl->text());
 }
+
+void MainWindow::toggleAnimation(bool checked) {
+    QSettings settings;
+    settings.setValue(SG_ANIMATIONS_PREFERENCE, checked);
+    animationAction->setChecked(checked);
+    if(ui->deviceImage->movie()) {
+        if(checked) {
+            ui->deviceImage->movie()->start();
+        } else {
+            ui->deviceImage->movie()->stop();
+        }
+    }
+}
