@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2011-2012 Yubico AB.  All rights reserved.
+Copyright (C) 2011-2013 Yubico AB.  All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
@@ -27,6 +27,9 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include "oathpage.h"
+#include "yubikeyutil.h"
+#include "yubikeyfinder.h"
+#include "yubikeywriter.h"
 #include "ui_oathpage.h"
 #include "ui/helpbox.h"
 #include "ui/confirmbox.h"
@@ -48,8 +51,9 @@ OathPage::OathPage(QWidget *parent) :
 
     m_customerPrefix = -1;
     memset(&m_pubId, 0, sizeof(m_pubId));
-    m_pubIdFormat = 0;
+    m_pubIdMUI = 0;
     m_ykConfig = 0;
+    m_keyPresent = false;
     clearState();
 
     //Connect pages
@@ -67,6 +71,18 @@ OathPage::OathPage(QWidget *parent) :
     connect(ui->advResetBtn, SIGNAL(clicked()),
             this, SLOT(resetAdvPage()));
 
+    connect(ui->advMovingFactorSeedTxt, SIGNAL(editingFinished()),
+            this, SLOT(on_advMovingFactorSeedTxt_editingFinished()));
+
+    connect(ui->advHotpLen6Radio, SIGNAL(clicked()),
+            this, SLOT(hotpLen_clicked()));
+    connect(ui->advHotpLen8Radio, SIGNAL(clicked()),
+            this, SLOT(hotpLen_clicked()));
+    connect(ui->quickHotpLen6Radio, SIGNAL(clicked()),
+            this, SLOT(hotpLen_clicked()));
+    connect(ui->quickHotpLen8Radio, SIGNAL(clicked()),
+            this, SLOT(hotpLen_clicked()));
+
     //Load settings
     loadSettings();
 
@@ -152,7 +168,6 @@ void OathPage::connectHelpButtons() {
 
     connect(ui->advConfigHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
     connect(ui->advParamGenSchemeHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
-    connect(ui->advConfigProtectionHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
     connect(ui->advPubIdHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
     connect(ui->advHotpLenHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
     connect(ui->advHotpParamsHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
@@ -166,7 +181,6 @@ void OathPage::connectHelpButtons() {
 
     mapper->setMapping(ui->advConfigHelpBtn, HelpBox::Help_ConfigurationSlot);
     mapper->setMapping(ui->advParamGenSchemeHelpBtn, HelpBox::Help_ParameterGeneration);
-    mapper->setMapping(ui->advConfigProtectionHelpBtn, HelpBox::Help_ConfigurationProtection);
     mapper->setMapping(ui->advPubIdHelpBtn, HelpBox::Help_OathPublicID);
     mapper->setMapping(ui->advHotpLenHelpBtn, HelpBox::Help_HotpLen);
     mapper->setMapping(ui->advHotpParamsHelpBtn, HelpBox::Help_HotpParam);
@@ -187,6 +201,7 @@ void OathPage::keyFound(bool found, bool* featuresMatrix) {
         if(m_state == State_Initial) {
             ui->quickWriteConfigBtn->setEnabled(true);
             ui->advWriteConfigBtn->setEnabled(true);
+            ui->advExportConfigBtn->setEnabled(true);
 
             if(!featuresMatrix[YubiKeyFinder::Feature_MultipleConfigurations]) {
                 ui->quickConfigSlot2Radio->setEnabled(false);
@@ -209,17 +224,23 @@ void OathPage::keyFound(bool found, bool* featuresMatrix) {
                 resetQuickPrefix();
             }
         } else if(this->currentIndex() == Page_Advanced &&
-                  m_state >= State_Programming_Multiple) {
+                  m_state >= State_Programming_Multiple &&
+                  m_keyPresent == false) {
             if(m_state == State_Programming_Multiple) {
                 ui->advWriteConfigBtn->setEnabled(true);
+                ui->advExportConfigBtn->setEnabled(true);
             } else {
-                writeAdvConfig();
+                writeAdvConfig(WRITE_CONFIG);
             }
         }
+        m_keyPresent = true;
     } else {
 
         ui->quickWriteConfigBtn->setEnabled(false);
         ui->advWriteConfigBtn->setEnabled(false);
+        ui->advExportConfigBtn->setEnabled(false);
+
+        m_keyPresent = false;
 
         if(m_state == State_Initial) {
             ui->quickConfigSlot2Radio->setEnabled(true);
@@ -262,69 +283,62 @@ void OathPage::updatePrefix() {
                      (unsigned char) (m_customerPrefix / 1000);
 
         if(m_customerPrefix > 0) {
-            unsigned short tmp;
-            tmp = m_customerPrefix % 1000;
-
-            m_pubId[2] = ((tmp / 100) << 4) | (( tmp % 100) / 10);
-            m_pubId[3] = (m_pubId[3] & 0xf) | ((tmp % 10) << 4);
+            // make room for the prefix..
+            m_pubIdMUI %= 99999;
+            int mui_part = m_customerPrefix % 1000;
+            m_pubIdMUI += mui_part * 100000;
         }
     }
 
     QString pubIdTxt;
+    QString muiTxt = QString::number(m_pubIdMUI).rightJustified(8, '0');
     QString pubIdModhexTxt = YubiKeyUtil::qstrModhexEncode(
-            m_pubId, OATH_HOTP_PUBLIC_ID_SIZE);
+            m_pubId, 2);
 
     switch(m_currentPage) {
     case Page_Quick:
-        fixBCD(m_pubId + 2, OATH_HOTP_TT_SIZE +  OATH_HOTP_MUI_SIZE);
-
-        pubIdTxt = YubiKeyUtil::qstrHexEncode(
-                m_pubId, OATH_HOTP_PUBLIC_ID_SIZE);
-
         ui->quickPrefixTxt->setText(pubIdModhexTxt.left(4));
-        ui->quickMUITxt->setText(pubIdTxt.right(8));
+        ui->quickMUITxt->setText(muiTxt);
         break;
 
     case Page_Advanced:
         switch(ui->advPubIdFormatCombo->currentIndex()){
         case OATH_FIXED_NUMERIC:
-            fixBCD(m_pubId, OATH_HOTP_PUBLIC_ID_SIZE);
-
+            fixBCD(m_pubId, 2);
             pubIdTxt = YubiKeyUtil::qstrHexEncode(
-                    m_pubId, OATH_HOTP_PUBLIC_ID_SIZE);
+                    m_pubId, 2);
 
             ui->advOMPTxt->setText(pubIdTxt.left(2));
             ui->advTTTxt->setText(pubIdTxt.mid(2, 2));
-            ui->advMUITxt->setText(pubIdTxt.right(8));
+            ui->advMUITxt->setText(muiTxt);
             break;
 
         case OATH_FIXED_MODHEX1:
-            fixBCD(m_pubId + 1, OATH_HOTP_TT_SIZE +  OATH_HOTP_MUI_SIZE);
-
+            fixBCD(m_pubId + 1, 1);
             pubIdTxt = YubiKeyUtil::qstrHexEncode(
-                    m_pubId, OATH_HOTP_PUBLIC_ID_SIZE);
-
+                    m_pubId, 2);
 
             ui->advOMPTxt->setText(pubIdModhexTxt.left(2));
             ui->advTTTxt->setText(pubIdTxt.mid(2, 2));
-            ui->advMUITxt->setText(pubIdTxt.right(8));
+            ui->advMUITxt->setText(muiTxt);
             break;
 
         case OATH_FIXED_MODHEX2:
-            fixBCD(m_pubId + 2, OATH_HOTP_MUI_SIZE);
-
-            pubIdTxt = YubiKeyUtil::qstrHexEncode(
-                    m_pubId, OATH_HOTP_PUBLIC_ID_SIZE);
-
             ui->advOMPTxt->setText(pubIdModhexTxt.left(2));
             ui->advTTTxt->setText(pubIdModhexTxt.mid(2, 2));
-            ui->advMUITxt->setText(pubIdTxt.right(8));
+            ui->advMUITxt->setText(muiTxt);
             break;
 
         case OATH_FIXED_MODHEX:
+            unsigned char tempMUI[4];
+            tempMUI[0] = (m_pubIdMUI >> 24) & 0xff;
+            tempMUI[1] = (m_pubIdMUI >> 16) & 0xff;
+            tempMUI[2] = (m_pubIdMUI >> 8) & 0xff;
+            tempMUI[3] = m_pubIdMUI & 0xff;
+            muiTxt = YubiKeyUtil::qstrModhexEncode(tempMUI, 4);
             ui->advOMPTxt->setText(pubIdModhexTxt.left(2));
             ui->advTTTxt->setText(pubIdModhexTxt.mid(2, 2));
-            ui->advMUITxt->setText(pubIdModhexTxt.right(8));
+            ui->advMUITxt->setText(muiTxt);
             break;
         }
 
@@ -352,6 +366,14 @@ void OathPage::loadSettings() {
     ui->advPubIdFormatCombo->setEnabled(customerPrefixFlag);
     ui->advOMPTxt->setEnabled(customerPrefixFlag);
     ui->advTTTxt->setEnabled(customerPrefixFlag);
+
+    ui->advExportConfigBtn->setVisible(settings.value(SG_EXPORT_PREFERENCE).toBool());
+
+    bool hotp8 = settings.value(SG_OATH_HOTP8).toBool();
+    ui->advHotpLen6Radio->setChecked(!hotp8);
+    ui->quickHotpLen6Radio->setChecked(!hotp8);
+    ui->advHotpLen8Radio->setChecked(hotp8);
+    ui->quickHotpLen8Radio->setChecked(hotp8);
 }
 
 /*
@@ -365,7 +387,6 @@ void OathPage::resetQuickPage() {
     }
 
     ui->quickPubIdCheck->setChecked(true);
-    ui->quickHotpLen6Radio->setChecked(true);
 
     on_quickResetBtn_clicked();
 }
@@ -422,7 +443,8 @@ void OathPage::on_quickMUITxt_editingFinished() {
 }
 
 void OathPage::on_quickMUIGenerateBtn_clicked() {
-    YubiKeyUtil::generateRandom(m_pubId + 2, OATH_HOTP_MUI_SIZE);
+    YubiKeyUtil::generateRandom((unsigned char*)&m_pubIdMUI, sizeof(m_pubIdMUI));
+    m_pubIdMUI %= 99999999;
     updatePrefix();
 }
 
@@ -485,6 +507,9 @@ void OathPage::writeQuickConfig() {
     //Programming mode...
     m_ykConfig->setProgrammingMode(YubiKeyConfig::Mode_OathHotp);
 
+    // set serial
+    m_ykConfig->setSerial(QString::number(YubiKeyFinder::getInstance()->serial()));
+
     //Configuration slot...
     int configSlot = 1;
     if(ui->quickConfigSlot2Radio->isChecked()) {
@@ -494,8 +519,7 @@ void OathPage::writeQuickConfig() {
 
     //Public ID...
     if(ui->quickPubIdCheck->isChecked()) {
-        QString pubIdTxt = YubiKeyUtil::qstrModhexEncode(m_pubId, OATH_HOTP_PUBLIC_ID_SIZE);
-        m_ykConfig->setPubIdTxt(pubIdTxt);
+        m_ykConfig->setPubIdTxt(getPublicId(true));
 
         //OATH Public ID Type...
         m_ykConfig->setOathFixedModhex2(true);
@@ -517,7 +541,7 @@ void OathPage::writeQuickConfig() {
     YubiKeyWriter::getInstance()->writeConfig(m_ykConfig);
 }
 
-void OathPage::quickConfigWritten(bool written, const QString &msg) {
+void OathPage::quickConfigWritten(bool written, __attribute__((unused)) const QString &msg) {
 
     disconnect(YubiKeyWriter::getInstance(), SIGNAL(configWritten(bool, const QString &)),
                this, SLOT(quickConfigWritten(bool, const QString &)));
@@ -546,6 +570,7 @@ void OathPage::quickConfigWritten(bool written, const QString &msg) {
 */
 void OathPage::resetAdvPage() {
     memset(&m_pubId, 0, sizeof(m_pubId));
+    m_pubIdMUI = 0;
 
     bool customerPrefixFlag = !(m_customerPrefix > 0);
 
@@ -557,7 +582,7 @@ void OathPage::resetAdvPage() {
     ui->advAutoProgramKeysCheck->setChecked(false);
     ui->advProgramMulKeysBox->setChecked(false);
 
-    ui->advConfigProtectionCombo->setCurrentIndex(0);
+    ui->advConfigProtectionBox->reset();
 
     ui->advPubIdCheck->setChecked(true);
     if(customerPrefixFlag) {
@@ -572,7 +597,6 @@ void OathPage::resetAdvPage() {
     ui->advTTTxt->setEnabled(customerPrefixFlag);
     on_advMUITxt_editingFinished();
 
-    ui->advHotpLen6Radio->setChecked(true);
     ui->advMovingFactorSeedCombo->setCurrentIndex(0);
     ui->advMovingFactorSeedTxt->setText(tr("0"));
     ui->advMovingFactorSeedTxt->setEnabled(false);
@@ -581,7 +605,6 @@ void OathPage::resetAdvPage() {
     on_advSecretKeyTxt_editingFinished();
 
     ui->advStopBtn->setEnabled(false);
-    ui->advResetBtn->setEnabled(false);
 }
 
 void OathPage::freezeAdvPage(bool freeze) {
@@ -592,6 +615,7 @@ void OathPage::freezeAdvPage(bool freeze) {
     ui->advKeyParamsBox->setEnabled(disable);
 
     ui->advWriteConfigBtn->setEnabled(disable);
+    ui->advExportConfigBtn->setEnabled(disable);
     ui->advStopBtn->setEnabled(!disable);
     ui->advResetBtn->setEnabled(disable);
     ui->advBackBtn->setEnabled(disable);
@@ -603,56 +627,10 @@ void OathPage::on_advProgramMulKeysBox_clicked(bool checked) {
     }
 }
 
-void OathPage::on_advConfigParamsCombo_currentIndexChanged(int index) {
+void OathPage::on_advConfigParamsCombo_currentIndexChanged(__attribute__((unused)) int index) {
     changeAdvConfigParams();
 }
 
-void OathPage::on_advConfigProtectionCombo_currentIndexChanged(int index) {
-    switch(index) {
-    case CONFIG_PROTECTION_DISABLED:
-        ui->advCurrentAccessCodeTxt->clear();
-        ui->advCurrentAccessCodeTxt->setEnabled(false);
-
-        ui->advNewAccessCodeTxt->clear();
-        ui->advNewAccessCodeTxt->setEnabled(false);
-        break;
-    case CONFIG_PROTECTION_ENABLE:
-        ui->advCurrentAccessCodeTxt->clear();
-        ui->advCurrentAccessCodeTxt->setEnabled(false);
-
-        on_advNewAccessCodeTxt_editingFinished();
-        ui->advNewAccessCodeTxt->setEnabled(true);
-        break;
-    case CONFIG_PROTECTION_DISABLE:
-    case CONFIG_PROTECTION_ENABLED:
-        on_advCurrentAccessCodeTxt_editingFinished();
-        ui->advCurrentAccessCodeTxt->setEnabled(true);
-
-        ui->advNewAccessCodeTxt->clear();
-        ui->advNewAccessCodeTxt->setEnabled(false);
-        break;
-    case CONFIG_PROTECTION_CHANGE:
-        on_advCurrentAccessCodeTxt_editingFinished();
-        ui->advCurrentAccessCodeTxt->setEnabled(true);
-
-        on_advNewAccessCodeTxt_editingFinished();
-        ui->advNewAccessCodeTxt->setEnabled(true);
-        break;
-    }
-}
-
-void OathPage::on_advCurrentAccessCodeTxt_editingFinished() {
-    QString txt = ui->advCurrentAccessCodeTxt->text();
-    YubiKeyUtil::qstrClean(&txt, (size_t)ACC_CODE_SIZE * 2);
-    ui->advCurrentAccessCodeTxt->setText(txt);
-}
-
-void OathPage::on_advNewAccessCodeTxt_editingFinished() {
-    QString txt = ui->advNewAccessCodeTxt->text();
-    YubiKeyUtil::qstrClean(&txt, (size_t)ACC_CODE_SIZE * 2);
-    ui->advNewAccessCodeTxt->setText(txt);
-}
-
 void OathPage::on_advPubIdCheck_stateChanged(int state) {
     bool disable = (state != 0);
 
@@ -666,49 +644,8 @@ void OathPage::on_advPubIdCheck_stateChanged(int state) {
     ui->advMUIGenerateBtn->setEnabled(disable);
 }
 
-void OathPage::on_advPubIdFormatCombo_currentIndexChanged(int index) {
-    updateAdvOMP(m_pubIdFormat);
-    updateAdvTT(m_pubIdFormat);
-    updateAdvMUI(m_pubIdFormat);
-
-    //        QRegExp modhex2("([b-lnrt-v ]){2}");
-    //        QRegExp modhex8("([b-lnrt-v ]){8}");
-
-    //        switch(index) {
-    //        case OATH_FIXED_MODHEX1:
-    //            ui->advOMPTxt->setInputMask("");
-    //            ui->advOMPTxt->setValidator(new QRegExpValidator(modhex2, this));
-    //            ui->advTTTxt->setInputMask(tr("00;"));
-    //            ui->advMUITxt->setInputMask(tr("00 00 00 00;"));
-    //            break;
-
-    //        case OATH_FIXED_MODHEX2:
-    //            ui->advOMPTxt->setInputMask("");
-    //            ui->advOMPTxt->setValidator(new QRegExpValidator(modhex2, this));
-    //            ui->advTTTxt->setInputMask("");
-    //            ui->advTTTxt->setValidator(new QRegExpValidator(modhex2, this));
-    //            ui->advMUITxt->setInputMask(tr("00 00 00 00;"));
-    //            break;
-
-    //        case OATH_FIXED_MODHEX:
-    //            ui->advOMPTxt->setInputMask("");
-    //            ui->advOMPTxt->setValidator(new QRegExpValidator(modhex2, this));
-    //            ui->advTTTxt->setInputMask("");
-    //            ui->advTTTxt->setValidator(new QRegExpValidator(modhex2, this));
-    //            ui->advMUITxt->setInputMask("");
-    //            ui->advMUITxt->setValidator(new QRegExpValidator(modhex8, this));
-    //            break;
-
-    //        default:
-    //            ui->advOMPTxt->setInputMask(tr("00;"));
-    //            ui->advTTTxt->setInputMask(tr("00;"));
-    //            ui->advMUITxt->setInputMask(tr("00 00 00 00;"));
-    //            break;
-    //        }
-
+void OathPage::on_advPubIdFormatCombo_currentIndexChanged(__attribute__((unused)) int index) {
     updatePrefix();
-
-    m_pubIdFormat = index;
 }
 
 void OathPage::updateAdvOMP(int index) {
@@ -761,20 +698,18 @@ void OathPage::on_advTTTxt_editingFinished() {
 void OathPage::updateAdvMUI(int index) {
     QString txt = ui->advMUITxt->text();
 
-    unsigned char buf[MAX_SIZE];
-    memset(buf, 0, sizeof(buf));
-    size_t bufLen = 0;
-
     if (index != OATH_FIXED_MODHEX) {
         YubiKeyUtil::qstrClean(&txt, OATH_HOTP_MUI_SIZE * 2, true);
-        YubiKeyUtil::qstrDecDecode(buf, &bufLen, txt);
+        m_pubIdMUI = txt.toInt();
     } else {
+        unsigned char buf[MAX_SIZE];
+        memset(buf, 0, sizeof(buf));
+        size_t bufLen = 0;
+
         YubiKeyUtil::qstrModhexClean(&txt, OATH_HOTP_MUI_SIZE * 2);
         YubiKeyUtil::qstrModhexDecode(buf, &bufLen, txt);
+        m_pubIdMUI = (buf[0] << 24) + (buf[1] << 16) + (buf[2] << 8) + (buf[3]);
     }
-
-    ui->advMUITxt->setText(txt);
-    memcpy(m_pubId + 2, buf, OATH_HOTP_MUI_SIZE);
 }
 
 void OathPage::on_advMUITxt_editingFinished() {
@@ -783,7 +718,8 @@ void OathPage::on_advMUITxt_editingFinished() {
 }
 
 void OathPage::on_advMUIGenerateBtn_clicked() {
-    YubiKeyUtil::generateRandom(m_pubId + 2, OATH_HOTP_MUI_SIZE);
+    YubiKeyUtil::generateRandom((unsigned char*)&m_pubIdMUI, sizeof(m_pubIdMUI));
+    m_pubIdMUI %= 99999999;
     updatePrefix();
 }
 
@@ -797,9 +733,9 @@ void OathPage::on_advMovingFactorSeedCombo_currentIndexChanged(int index) {
         ui->advMovingFactorSeedTxt->setEnabled(true);
         break;
     case MOVING_FACTOR_RAND:
-        unsigned char tmp;
+        unsigned int tmp;
         YubiKeyUtil::generateRandom((unsigned char *) &tmp, sizeof(tmp));
-        ui->advMovingFactorSeedTxt->setText(QString::number((unsigned int)tmp));
+        ui->advMovingFactorSeedTxt->setText(QString::number(tmp));
         on_advMovingFactorSeedTxt_editingFinished();
         ui->advMovingFactorSeedTxt->setEnabled(true);
         break;
@@ -853,7 +789,22 @@ void OathPage::on_advWriteConfigBtn_clicked() {
         m_state = State_Programming;
     }
 
-    writeAdvConfig();
+    writeAdvConfig(WRITE_CONFIG);
+}
+
+void OathPage::on_advExportConfigBtn_clicked() {
+    emit showStatusMessage(NULL, -1);
+
+    //Validate settings
+    if(!validateAdvSettings()) {
+        return;
+    }
+
+    clearState();
+
+    freezeAdvPage(true);
+
+    writeAdvConfig(EXPORT_CONFIG);
 }
 
 void OathPage::on_advStopBtn_clicked() {
@@ -863,7 +814,7 @@ void OathPage::on_advStopBtn_clicked() {
 }
 
 void OathPage::stopAdvConfigWritting() {
-    qDebug() << "Stopping adv confgiuration writing...";
+    qDebug() << "Stopping adv configuration writing...";
 
     if(m_state >= State_Programming_Multiple) {
         ui->advStopBtn->setEnabled(true);
@@ -902,19 +853,15 @@ void OathPage::changeAdvConfigParams() {
         switch(idScheme) {
         case GEN_SCHEME_INCR:
             {
-                size_t pubIdLen = sizeof(m_pubId)/sizeof(m_pubId[0]);
-                for (int i = pubIdLen; i--; ) {
-                    if (++m_pubId[i]) {
-                        break;
-                    }
-                }
+                m_pubIdMUI++;
                 updatePrefix();
             }
             break;
 
         case GEN_SCHEME_RAND:
+            YubiKeyUtil::generateRandom((unsigned char*)&m_pubIdMUI, sizeof(m_pubIdMUI));
+            m_pubIdMUI %= 99999999;
 
-            YubiKeyUtil::generateRandom(m_pubId, OATH_HOTP_PUBLIC_ID_SIZE);
             updatePrefix();
             break;
         }
@@ -922,9 +869,9 @@ void OathPage::changeAdvConfigParams() {
 
     //HOTP Moving Factor Seed...
     if(ui->advMovingFactorSeedCombo->currentIndex() == MOVING_FACTOR_RAND) {
-        unsigned char tmp;
+        unsigned int tmp;
         YubiKeyUtil::generateRandom((unsigned char *) &tmp, sizeof(tmp));
-        ui->advMovingFactorSeedTxt->setText(QString::number((unsigned int)tmp));
+        ui->advMovingFactorSeedTxt->setText(QString::number(tmp));
         on_advMovingFactorSeedTxt_editingFinished();
     }
 
@@ -963,28 +910,37 @@ bool OathPage::validateAdvSettings() {
         }
     }
 
-    //Check if logging is disabled and
-    //configuration protection is being enabled
-    if(!settings.value(SG_ENABLE_CONF_PROTECTION).toBool() &&
-       !YubiKeyLogger::isLogging() &&
-       ui->advConfigProtectionCombo->currentIndex() == CONFIG_PROTECTION_ENABLE) {
-        //Confirm from client
-        ConfirmBox confirm(this);
-        confirm.setConfirmIndex(ConfirmBox::Confirm_ConfigurationProtection);
-        int ret = confirm.exec();
-
-        switch (ret) {
-        case 1:     //Yes
-            break;
-        default:    //No
-            return false;
-        }
+    if(!ui->advConfigProtectionBox->checkConfirm()) {
+        return false;
     }
 
     return true;
 }
 
-void OathPage::writeAdvConfig() {
+QString OathPage::getPublicId(bool bcd) {
+  unsigned char pubId[6];
+  memcpy(pubId, m_pubId, 2);
+
+  if(bcd) {
+    int part = m_pubIdMUI / 1000000;
+    pubId[2] = ((part / 10) << 4) + part % 10;
+    part = m_pubIdMUI / 10000 % 100;
+    pubId[3] = ((part / 10) << 4) + part % 10;
+    part = m_pubIdMUI / 100 % 100;
+    pubId[4] = ((part / 10) << 4) + part % 10;
+    part = m_pubIdMUI % 100;
+    pubId[5] = ((part / 10) << 4) + part % 10;
+  } else {
+    pubId[2] = (m_pubIdMUI >> 24) & 0xff;
+    pubId[3] = (m_pubIdMUI >> 16) & 0xff;
+    pubId[4] = (m_pubIdMUI >> 8) & 0xff;
+    pubId[5] = m_pubIdMUI & 0xff;
+  }
+  QString pubIdTxt = YubiKeyUtil::qstrModhexEncode(pubId, 6);
+  return pubIdTxt;
+}
+
+void OathPage::writeAdvConfig(int mode) {
     qDebug() << "Writing configuration...";
 
     //Disable stop button while configuration is being written
@@ -1002,6 +958,9 @@ void OathPage::writeAdvConfig() {
     //Programming mode...
     m_ykConfig->setProgrammingMode(YubiKeyConfig::Mode_OathHotp);
 
+    // set serial
+    m_ykConfig->setSerial(QString::number(YubiKeyFinder::getInstance()->serial()));
+
     //Configuration slot...
     int configSlot = 1;
     if(ui->advConfigSlot2Radio->isChecked()) {
@@ -1011,8 +970,8 @@ void OathPage::writeAdvConfig() {
 
     //Public ID...
     if(ui->advPubIdCheck->isChecked()) {
-        QString pubIdTxt = YubiKeyUtil::qstrModhexEncode(m_pubId, OATH_HOTP_PUBLIC_ID_SIZE);
-        m_ykConfig->setPubIdTxt(pubIdTxt);
+        bool bcd = ui->advPubIdFormatCombo->currentIndex() < 3 ? true : false;
+        m_ykConfig->setPubIdTxt(getPublicId(bcd));
 
         //OATH Public ID Type...
         switch(ui->advPubIdFormatCombo->currentIndex()) {
@@ -1036,29 +995,31 @@ void OathPage::writeAdvConfig() {
             YubiKeyFinder::Feature_MovingFactor)) {
 
         m_ykConfig->setOathMovingFactorSeed(
-                ui->advMovingFactorSeedTxt->text().toUShort());
+                ui->advMovingFactorSeedTxt->text().toUInt());
     }
 
     //Secret Key...
     m_ykConfig->setSecretKeyTxt(ui->advSecretKeyTxt->text());
 
     //Configuration protection...
-    //Current Access Code...
-    m_ykConfig->setCurrentAccessCodeTxt(ui->advCurrentAccessCodeTxt->text());
+    m_ykConfig->setCurrentAccessCodeTxt(
+        ui->advConfigProtectionBox->currentAccessCode());
+    m_ykConfig->setNewAccessCodeTxt(
+        ui->advConfigProtectionBox->newAccessCode(),
+        ui->advConfigProtectionBox->newAccMode());
 
-    //New Access Code...
-    if(ui->advConfigProtectionCombo->currentIndex()
-        == CONFIG_PROTECTION_DISABLE){
-        m_ykConfig->setNewAccessCodeTxt(ACCESS_CODE_DEFAULT);
-    } else {
-        m_ykConfig->setNewAccessCodeTxt(ui->advNewAccessCodeTxt->text());
+    if(mode == WRITE_CONFIG) {
+        //Write
+        connect(YubiKeyWriter::getInstance(), SIGNAL(configWritten(bool, const QString &)),
+                this, SLOT(advConfigWritten(bool, const QString &)));
+
+        YubiKeyWriter::getInstance()->writeConfig(m_ykConfig);
+    } else if(mode == EXPORT_CONFIG) {
+        connect(YubiKeyWriter::getInstance(), SIGNAL(configWritten(bool, const QString &)),
+                this, SLOT(advConfigExported(bool, const QString &)));
+
+        YubiKeyWriter::getInstance()->exportConfig(m_ykConfig);
     }
-
-    //Write
-    connect(YubiKeyWriter::getInstance(), SIGNAL(configWritten(bool, const QString &)),
-            this, SLOT(advConfigWritten(bool, const QString &)));
-
-    YubiKeyWriter::getInstance()->writeConfig(m_ykConfig);
 }
 
 void OathPage::advConfigWritten(bool written, const QString &msg) {
@@ -1103,6 +1064,28 @@ void OathPage::advConfigWritten(bool written, const QString &msg) {
     stopAdvConfigWritting();
 }
 
+void OathPage::advConfigExported(bool written, const QString &msg) {
+    disconnect(YubiKeyWriter::getInstance(), SIGNAL(configWritten(bool, const QString &)),
+               this, SLOT(advConfigExported(bool, const QString &)));
+
+    QString message;
+
+    if(written && m_ykConfig != 0) {
+        qDebug() << "Configuration exported....";
+
+        message = KEY_EXPORTED;
+
+        showStatusMessage(message, 0);
+    } else {
+        qDebug() << "Configuration could not be exported....";
+
+        message = msg;
+    }
+
+    m_ready = false;
+    stopAdvConfigWritting();
+}
+
 void OathPage::advUpdateResults(bool written, const QString &msg) {
     int row = 0;
 
@@ -1153,3 +1136,29 @@ void OathPage::advUpdateResults(bool written, const QString &msg) {
     ui->advResultsWidget->resizeColumnsToContents();
     ui->advResultsWidget->resizeRowsToContents();
 }
+
+void OathPage::hotpLen_clicked() {
+    QSettings settings;
+    QRadioButton *button = ui->advHotpLen8Radio;
+    QRadioButton *button2 = ui->quickHotpLen8Radio;
+    if(m_currentPage == Page_Quick) {
+        button = ui->quickHotpLen8Radio;
+        button2 = ui->advHotpLen8Radio;
+    }
+    if(button->isChecked()) {
+        settings.setValue(SG_OATH_HOTP8, true);
+    } else {
+        settings.setValue(SG_OATH_HOTP8, false);
+    }
+    button2->toggle();
+}
+
+void OathPage::setCurrentSlot(int slot) {
+    if(m_currentPage == Page_Advanced) {
+        ui->advConfigSlot1Radio->setChecked(slot == 1);
+        ui->advConfigSlot2Radio->setChecked(slot == 2);
+    } else if(m_currentPage == Page_Quick) {
+        ui->quickConfigSlot1Radio->setChecked(slot == 1);
+        ui->quickConfigSlot2Radio->setChecked(slot == 2);
+    }
+}
