@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2011-2012 Yubico AB.  All rights reserved.
+Copyright (C) 2011-2013 Yubico AB.  All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
@@ -27,6 +27,9 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include "otppage.h"
+#include "yubikeywriter.h"
+#include "yubikeyutil.h"
+#include "yubikeyfinder.h"
 #include "ui_otppage.h"
 #include "ui/helpbox.h"
 #include "ui/confirmbox.h"
@@ -42,6 +45,7 @@ OtpPage::OtpPage(QWidget *parent) :
 
     m_customerPrefix = -1;
     m_ykConfig = 0;
+    m_keyPresent = false;
     clearState();
 
     //Connect pages
@@ -148,7 +152,6 @@ void OtpPage::connectHelpButtons() {
 
     connect(ui->advConfigHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
     connect(ui->advParamGenSchemeHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
-    connect(ui->advConfigProtectionHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
     connect(ui->advPubIdHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
     connect(ui->advPvtIdHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
     connect(ui->advSecretKeyHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
@@ -161,7 +164,6 @@ void OtpPage::connectHelpButtons() {
 
     mapper->setMapping(ui->advConfigHelpBtn, HelpBox::Help_ConfigurationSlot);
     mapper->setMapping(ui->advParamGenSchemeHelpBtn, HelpBox::Help_ParameterGeneration);
-    mapper->setMapping(ui->advConfigProtectionHelpBtn, HelpBox::Help_ConfigurationProtection);
     mapper->setMapping(ui->advPubIdHelpBtn, HelpBox::Help_PublicID);
     mapper->setMapping(ui->advPvtIdHelpBtn, HelpBox::Help_PrivateID);
     mapper->setMapping(ui->advSecretKeyHelpBtn, HelpBox::Help_SecretKey);
@@ -181,6 +183,7 @@ void OtpPage::keyFound(bool found, bool* featuresMatrix) {
         if(m_state == State_Initial) {
             ui->quickWriteConfigBtn->setEnabled(true);
             ui->advWriteConfigBtn->setEnabled(true);
+            ui->advExportConfigBtn->setEnabled(true);
 
             if(!featuresMatrix[YubiKeyFinder::Feature_MultipleConfigurations]) {
                 ui->quickConfigSlot2Radio->setEnabled(false);
@@ -190,25 +193,29 @@ void OtpPage::keyFound(bool found, bool* featuresMatrix) {
                 ui->advConfigSlot2Radio->setEnabled(true);
             }
         } else if(this->currentIndex() == Page_Advanced &&
-                  m_state >= State_Programming_Multiple) {
+                  m_state >= State_Programming_Multiple &&
+                  m_keyPresent == false) {
+            if(m_keysProgrammedCtr > 0 && !m_ready) {
+                changeAdvConfigParams();
+            }
             if(m_state == State_Programming_Multiple) {
                 ui->advWriteConfigBtn->setEnabled(true);
+                ui->advExportConfigBtn->setEnabled(true);
             } else {
-                writeAdvConfig();
+                writeAdvConfig(WRITE_CONFIG);
             }
         }
+        m_keyPresent = true;
     } else {
         ui->quickWriteConfigBtn->setEnabled(false);
         ui->advWriteConfigBtn->setEnabled(false);
+        ui->advExportConfigBtn->setEnabled(false);
+
+        m_keyPresent = false;
 
         if(m_state == State_Initial) {
             ui->quickConfigSlot2Radio->setEnabled(true);
             ui->advConfigSlot2Radio->setEnabled(true);
-        } else if(this->currentIndex() == Page_Advanced &&
-                  m_state >= State_Programming_Multiple) {
-            if(m_keysProgrammedCtr > 0 && !m_ready) {
-                changeAdvConfigParams();
-            }
         }
     }
 }
@@ -235,10 +242,9 @@ void OtpPage::loadSettings() {
         QByteArray prefix;
 
         // As per old tool
-        prefix.resize(3);
-        prefix[0] = YUBICO_OTP_CUSTOMER_PREFIX_CODE;
-        prefix[1] = (unsigned char) (m_customerPrefix >> 8U);
-        prefix[2] = (unsigned char) m_customerPrefix;
+        prefix.resize(2);
+        prefix[0] = (unsigned char) (m_customerPrefix >> 8U);
+        prefix[1] = (unsigned char) m_customerPrefix;
 
         char pubIdPrefix[prefix.size() * 2 + 1];
         size_t pubIdPrefixLen = 0;
@@ -258,8 +264,9 @@ void OtpPage::loadSettings() {
 
         m_pubIdPrefix = QString("");
     }
-}
 
+    ui->advExportConfigBtn->setVisible(settings.value(SG_EXPORT_PREFERENCE).toBool());
+}
 
 /*
  Quick Page handling
@@ -360,7 +367,6 @@ void OtpPage::writeQuickConfig() {
     //Write configuration
     ui->quickWriteConfigBtn->setEnabled(false);
     ui->quickUploadBtn->setEnabled(false);
-    ui->quickResetBtn->setEnabled(false);
     ui->quickBackBtn->setEnabled(false);
 
     if(m_ykConfig != 0) {
@@ -372,6 +378,9 @@ void OtpPage::writeQuickConfig() {
     //Programming mode...
     m_ykConfig->setProgrammingMode(YubiKeyConfig::Mode_YubicoOtp);
 
+    // set serial
+    m_ykConfig->setSerial(QString::number(YubiKeyFinder::getInstance()->serial()));
+
     //Configuration slot...
     int configSlot = 1;
     if(ui->quickConfigSlot2Radio->isChecked()) {
@@ -404,12 +413,11 @@ void OtpPage::writeQuickConfig() {
     YubiKeyWriter::getInstance()->writeConfig(m_ykConfig);
 }
 
-void OtpPage::quickConfigWritten(bool written, const QString &msg) {
+void OtpPage::quickConfigWritten(bool written, __attribute__((unused)) const QString &msg) {
     disconnect(YubiKeyWriter::getInstance(), SIGNAL(configWritten(bool, const QString &)),
                this, SLOT(quickConfigWritten(bool, const QString &)));
 
     ui->quickWriteConfigBtn->setEnabled(true);
-    ui->quickResetBtn->setEnabled(true);
     ui->quickBackBtn->setEnabled(true);
 
     if(written && m_ykConfig != 0) {
@@ -426,7 +434,12 @@ void OtpPage::quickConfigWritten(bool written, const QString &msg) {
 }
 
 void OtpPage::uploadQuickConfig() {
-    QDesktopServices::openUrl(QUrl(m_uploadUrl));
+    m_tempFile.setFileTemplate(QDir::tempPath() + "/ykpers.XXXXXX.html");
+    m_tempFile.open();
+    m_tempFile.write(m_uploadUrl.toAscii(), m_uploadUrl.length());
+    m_tempFile.close();
+    qDebug() << "temporary file for upload is: " << m_tempFile.fileName();
+    QDesktopServices::openUrl(QUrl::fromLocalFile(m_tempFile.fileName()));
 }
 
 /*
@@ -438,14 +451,15 @@ void OtpPage::resetAdvPage() {
         ui->advConfigSlot2Radio->setChecked(true);
     }
 
-    ui->advConfigParamsCombo->setCurrentIndex(0);
+    ui->advConfigParamsCombo->setCurrentIndex(2);
     ui->advAutoProgramKeysCheck->setChecked(false);
     ui->advProgramMulKeysBox->setChecked(false);
 
-    ui->advConfigProtectionCombo->setCurrentIndex(0);
+    ui->advConfigProtectionBox->reset();
 
     ui->advPubIdCheck->setChecked(true);
     ui->advPubIdTxt->clear();
+    set_advPubId_default();
     on_advPubIdTxt_editingFinished();
     ui->advPubIdLenBox->setValue(PUBLIC_ID_DEFAULT_SIZE);
     if(m_customerPrefix > 0) {
@@ -462,7 +476,6 @@ void OtpPage::resetAdvPage() {
     on_advSecretKeyTxt_editingFinished();
 
     ui->advStopBtn->setEnabled(false);
-    ui->advResetBtn->setEnabled(false);
 }
 
 void OtpPage::freezeAdvPage(bool freeze) {
@@ -473,8 +486,8 @@ void OtpPage::freezeAdvPage(bool freeze) {
     ui->advKeyParamsBox->setEnabled(disable);
 
     ui->advWriteConfigBtn->setEnabled(disable);
+    ui->advExportConfigBtn->setEnabled(disable);
     ui->advStopBtn->setEnabled(!disable);
-    ui->advResetBtn->setEnabled(disable);
     ui->advBackBtn->setEnabled(disable);
 }
 
@@ -484,55 +497,6 @@ void OtpPage::on_advProgramMulKeysBox_clicked(bool checked) {
     }
 }
 
-void OtpPage::on_advConfigParamsCombo_currentIndexChanged(int index) {
-    changeAdvConfigParams();
-}
-
-void OtpPage::on_advConfigProtectionCombo_currentIndexChanged(int index) {
-    switch(index) {
-    case CONFIG_PROTECTION_DISABLED:
-        ui->advCurrentAccessCodeTxt->clear();
-        ui->advCurrentAccessCodeTxt->setEnabled(false);
-
-        ui->advNewAccessCodeTxt->clear();
-        ui->advNewAccessCodeTxt->setEnabled(false);
-        break;
-    case CONFIG_PROTECTION_ENABLE:
-        ui->advCurrentAccessCodeTxt->clear();
-        ui->advCurrentAccessCodeTxt->setEnabled(false);
-
-        on_advNewAccessCodeTxt_editingFinished();
-        ui->advNewAccessCodeTxt->setEnabled(true);
-        break;
-    case CONFIG_PROTECTION_DISABLE:
-    case CONFIG_PROTECTION_ENABLED:
-        on_advCurrentAccessCodeTxt_editingFinished();
-        ui->advCurrentAccessCodeTxt->setEnabled(true);
-
-        ui->advNewAccessCodeTxt->clear();
-        ui->advNewAccessCodeTxt->setEnabled(false);
-        break;
-    case CONFIG_PROTECTION_CHANGE:
-        on_advCurrentAccessCodeTxt_editingFinished();
-        ui->advCurrentAccessCodeTxt->setEnabled(true);
-
-        on_advNewAccessCodeTxt_editingFinished();
-        ui->advNewAccessCodeTxt->setEnabled(true);
-        break;
-    }
-}
-
-void OtpPage::on_advCurrentAccessCodeTxt_editingFinished() {
-    QString txt = ui->advCurrentAccessCodeTxt->text();
-    YubiKeyUtil::qstrClean(&txt, (size_t)ACC_CODE_SIZE * 2);
-    ui->advCurrentAccessCodeTxt->setText(txt);
-}
-
-void OtpPage::on_advNewAccessCodeTxt_editingFinished() {
-    QString txt = ui->advNewAccessCodeTxt->text();
-    YubiKeyUtil::qstrClean(&txt, (size_t)ACC_CODE_SIZE * 2);
-    ui->advNewAccessCodeTxt->setText(txt);
-}
 
 void OtpPage::on_advPubIdCheck_stateChanged(int state) {
     bool disable = (state != 0);
@@ -543,13 +507,35 @@ void OtpPage::on_advPubIdCheck_stateChanged(int state) {
     }
 }
 
+void OtpPage::set_advPubId_default() {
+    QString txt = "cccccc";
+    unsigned int serial = YubiKeyFinder::getInstance()->serial();
+
+    //Convert serial number to modhex
+    unsigned char buf[16];
+    memset(buf, 0, sizeof(buf));
+    size_t bufLen = 0;
+
+    QString tmp = QString::number(serial, 16);
+    size_t len = tmp.length();
+    if(len % 2 != 0) {
+      len++;
+    }
+    YubiKeyUtil::qstrClean(&tmp, (size_t)len, true);
+    YubiKeyUtil::qstrHexDecode(buf, &bufLen, tmp);
+
+    txt.append(YubiKeyUtil::qstrModhexEncode(buf, bufLen));
+    ui->advPubIdTxt->setText(txt);
+    on_advPubIdTxt_editingFinished();
+}
+
 void OtpPage::on_advPubIdTxt_editingFinished() {
     QString txt = ui->advPubIdTxt->text();
     int len = ui->advPubIdLenBox->value() * 2;
     YubiKeyUtil::qstrModhexClean(&txt, (size_t)len);
 
     if(m_customerPrefix > 0) {
-        txt.replace(0, 6, m_pubIdPrefix);
+        txt.replace(0, 4, m_pubIdPrefix);
     }
     ui->advPubIdTxt->setText(txt);
     len = txt.length();
@@ -598,6 +584,20 @@ void OtpPage::on_advSecretKeyGenerateBtn_clicked() {
             YubiKeyUtil::generateRandomHex((size_t)KEY_SIZE * 2));
 }
 
+void OtpPage::on_advExportConfigBtn_clicked() {
+    qDebug() << "foo";
+    //Validate settings
+    if(!validateAdvSettings()) {
+        return;
+    }
+
+    clearState();
+
+    freezeAdvPage(true);
+
+    writeAdvConfig(EXPORT_CONFIG);
+}
+
 void OtpPage::on_advWriteConfigBtn_clicked() {
     emit showStatusMessage(NULL, -1);
 
@@ -627,7 +627,7 @@ void OtpPage::on_advWriteConfigBtn_clicked() {
         m_state = State_Programming;
     }
 
-    writeAdvConfig();
+    writeAdvConfig(WRITE_CONFIG);
 }
 
 void OtpPage::on_advStopBtn_clicked() {
@@ -637,7 +637,7 @@ void OtpPage::on_advStopBtn_clicked() {
 }
 
 void OtpPage::stopAdvConfigWritting() {
-    qDebug() << "Stopping adv confgiuration writing...";
+    qDebug() << "Stopping adv configuration writing...";
 
     if(m_state >= State_Programming_Multiple) {
         ui->advStopBtn->setEnabled(true);
@@ -648,11 +648,10 @@ void OtpPage::stopAdvConfigWritting() {
     clearState();
 
     freezeAdvPage(false);
-    ui->advResetBtn->setEnabled(true);
 }
 
 void OtpPage::changeAdvConfigParams() {
-    qDebug() << "Changing adv confgiuration prams...";
+    qDebug() << "Changing adv configuration params...";
 
     int index = ui->advConfigParamsCombo->currentIndex();
     int idScheme = GEN_SCHEME_FIXED;
@@ -671,23 +670,32 @@ void OtpPage::changeAdvConfigParams() {
         idScheme = GEN_SCHEME_RAND;
         secretScheme = GEN_SCHEME_RAND;
         break;
+
+    case SCHEME_ID_FROM_SERIAL_RAND_SECRET:
+        idScheme = GEN_SCHEME_SERIAL;
+        secretScheme = GEN_SCHEME_RAND;
+        break;
     }
 
     //Public ID...
     if(ui->advPubIdCheck->isChecked()) {
-        QString pubIdTxt = YubiKeyUtil::getNextModhex(
+        if(idScheme == GEN_SCHEME_SERIAL) {
+            set_advPubId_default();
+        } else {
+            QString pubIdTxt = YubiKeyUtil::getNextModhex(
                 ui->advPubIdLenBox->value() * 2,
                 ui->advPubIdTxt->text(), idScheme);
 
-        ui->advPubIdTxt->setText(pubIdTxt);
-        on_advPubIdTxt_editingFinished();
+            ui->advPubIdTxt->setText(pubIdTxt);
+            on_advPubIdTxt_editingFinished();
+        }
     }
 
     //Private ID...
     if(ui->advPvtIdCheck->isChecked()) {
         QString pvtIdTxt = YubiKeyUtil::getNextHex(
                 UID_SIZE * 2,
-                ui->advPvtIdTxt->text(), idScheme);
+                ui->advPvtIdTxt->text(), secretScheme);
         ui->advPvtIdTxt->setText(pvtIdTxt);
     }
 
@@ -726,23 +734,6 @@ bool OtpPage::validateAdvSettings() {
         }
     }
 
-    //Check if logging is disabled and
-    //configuration protection is being enabled
-    if(!settings.value(SG_ENABLE_CONF_PROTECTION).toBool() &&
-       !YubiKeyLogger::isLogging() &&
-       ui->advConfigProtectionCombo->currentIndex() == CONFIG_PROTECTION_ENABLE) {
-        //Confirm from client
-        ConfirmBox confirm(this);
-        confirm.setConfirmIndex(ConfirmBox::Confirm_ConfigurationProtection);
-        int ret = confirm.exec();
-
-        switch (ret) {
-        case 1:     //Yes
-            break;
-        default:    //No
-            return false;
-        }
-    }
 
     //Check if public id length is 6
     if (!settings.value(SG_DIFF_PUBLIC_ID_LEN).toBool() &&
@@ -761,10 +752,14 @@ bool OtpPage::validateAdvSettings() {
         }
     }
 
+    if(!ui->advConfigProtectionBox->checkConfirm()) {
+        return false;
+    }
+
     return true;
 }
 
-void OtpPage::writeAdvConfig() {
+void OtpPage::writeAdvConfig(int mode) {
     qDebug() << "Writing configuration...";
 
     //Disable stop button while configuration is being written
@@ -781,6 +776,9 @@ void OtpPage::writeAdvConfig() {
     //Programming mode...
     m_ykConfig->setProgrammingMode(YubiKeyConfig::Mode_YubicoOtp);
 
+    // set serial
+    m_ykConfig->setSerial(QString::number(YubiKeyFinder::getInstance()->serial()));
+
     //Configuration slot...
     int configSlot = 1;
     if(ui->advConfigSlot2Radio->isChecked()) {
@@ -802,22 +800,24 @@ void OtpPage::writeAdvConfig() {
     m_ykConfig->setSecretKeyTxt(ui->advSecretKeyTxt->text());
 
     //Configuration protection...
-    //Current Access Code...
-    m_ykConfig->setCurrentAccessCodeTxt(ui->advCurrentAccessCodeTxt->text());
+    m_ykConfig->setCurrentAccessCodeTxt(
+        ui->advConfigProtectionBox->currentAccessCode());
+    m_ykConfig->setNewAccessCodeTxt(
+        ui->advConfigProtectionBox->newAccessCode(),
+        ui->advConfigProtectionBox->newAccMode());
 
-    //New Access Code...
-    if(ui->advConfigProtectionCombo->currentIndex()
-        == CONFIG_PROTECTION_DISABLE){
-        m_ykConfig->setNewAccessCodeTxt(ACCESS_CODE_DEFAULT);
-    } else {
-        m_ykConfig->setNewAccessCodeTxt(ui->advNewAccessCodeTxt->text());
+    if(mode == WRITE_CONFIG) {
+        //Write
+        connect(YubiKeyWriter::getInstance(), SIGNAL(configWritten(bool, const QString &)),
+                this, SLOT(advConfigWritten(bool, const QString &)));
+
+        YubiKeyWriter::getInstance()->writeConfig(m_ykConfig);
+    } else if(mode == EXPORT_CONFIG) {
+        connect(YubiKeyWriter::getInstance(), SIGNAL(configWritten(bool, const QString &)),
+                this, SLOT(advConfigExported(bool, const QString &)));
+
+        YubiKeyWriter::getInstance()->exportConfig(m_ykConfig);
     }
-
-    //Write
-    connect(YubiKeyWriter::getInstance(), SIGNAL(configWritten(bool, const QString &)),
-            this, SLOT(advConfigWritten(bool, const QString &)));
-
-    YubiKeyWriter::getInstance()->writeConfig(m_ykConfig);
 }
 
 void OtpPage::advConfigWritten(bool written, const QString &msg) {
@@ -857,6 +857,33 @@ void OtpPage::advConfigWritten(bool written, const QString &msg) {
     stopAdvConfigWritting();
 }
 
+void OtpPage::advConfigExported(bool written, const QString &msg) {
+    disconnect(YubiKeyWriter::getInstance(), SIGNAL(configWritten(bool, const QString &)),
+               this, SLOT(advConfigExported(bool, const QString &)));
+
+    QString message;
+
+    if(written && m_ykConfig != 0) {
+        qDebug() << "Configuration exported....";
+
+        QString keyDetail("");
+        if(ui->advPubIdCheck->isChecked()) {
+            keyDetail = tr(" (Public ID: %1)").arg(m_ykConfig->pubIdTxt());
+        }
+
+        message = KEY_EXPORTED;
+
+        showStatusMessage(message, 0);
+    } else {
+        qDebug() << "Configuration could not be exported....";
+
+        message = msg;
+    }
+
+    m_ready = false;
+    stopAdvConfigWritting();
+}
+
 void OtpPage::advUpdateResults(bool written, const QString &msg) {
     int row = 0;
 
@@ -906,3 +933,13 @@ void OtpPage::advUpdateResults(bool written, const QString &msg) {
     ui->advResultsWidget->resizeColumnsToContents();
     ui->advResultsWidget->resizeRowsToContents();
 }
+
+void OtpPage::setCurrentSlot(int slot) {
+    if(m_currentPage == Page_Advanced) {
+        ui->advConfigSlot1Radio->setChecked(slot == 1);
+        ui->advConfigSlot2Radio->setChecked(slot == 2);
+    } else if(m_currentPage == Page_Quick) {
+        ui->quickConfigSlot1Radio->setChecked(slot == 1);
+        ui->quickConfigSlot2Radio->setChecked(slot == 2);
+    }
+}
