@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2011-2012 Yubico AB.  All rights reserved.
+Copyright (C) 2011-2013 Yubico AB.  All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
@@ -27,6 +27,9 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include "staticpage.h"
+#include "yubikeyfinder.h"
+#include "yubikeyutil.h"
+#include "yubikeywriter.h"
 #include "ui_staticpage.h"
 #include "ui/helpbox.h"
 #include "ui/confirmbox.h"
@@ -42,6 +45,7 @@ StaticPage::StaticPage(QWidget *parent) :
     ui->setupUi(this);
 
     m_ykConfig = 0;
+    m_keyPresent = false;
     clearState();
 
     //Connect pages
@@ -61,6 +65,9 @@ StaticPage::StaticPage(QWidget *parent) :
 
     ui->quickResultsWidget->resizeColumnsToContents();
     ui->advResultsWidget->resizeColumnsToContents();
+
+    QRegExp rx("^[a-f0-9]{0,72}$");
+    ui->quickScanCodesTxt->setValidator(new QRegExpValidator(rx, this));
 }
 
 StaticPage::~StaticPage() {
@@ -138,24 +145,20 @@ void StaticPage::connectHelpButtons() {
 
     //Connect the clicked signal with the QSignalMapper
     connect(ui->quickConfigHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
-    connect(ui->quickConfigProtectionHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
     connect(ui->quickStaticScanCodeHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
 
     connect(ui->advConfigHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
     connect(ui->advParamGenSchemeHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
-    connect(ui->advConfigProtectionHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
     connect(ui->advPubIdHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
     connect(ui->advPvtIdHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
     connect(ui->advSecretKeyHelpBtn, SIGNAL(clicked()), mapper, SLOT(map()));
 
     //Set a value for each button
     mapper->setMapping(ui->quickConfigHelpBtn, HelpBox::Help_ConfigurationSlot);
-    mapper->setMapping(ui->quickConfigProtectionHelpBtn, HelpBox::Help_ConfigurationProtection);
     mapper->setMapping(ui->quickStaticScanCodeHelpBtn, HelpBox::Help_StaticScanCode);
 
     mapper->setMapping(ui->advConfigHelpBtn, HelpBox::Help_ConfigurationSlot);
     mapper->setMapping(ui->advParamGenSchemeHelpBtn, HelpBox::Help_ParameterGeneration);
-    mapper->setMapping(ui->advConfigProtectionHelpBtn, HelpBox::Help_ConfigurationProtection);
     mapper->setMapping(ui->advPubIdHelpBtn, HelpBox::Help_PublicID);
     mapper->setMapping(ui->advPvtIdHelpBtn, HelpBox::Help_PrivateID);
     mapper->setMapping(ui->advSecretKeyHelpBtn, HelpBox::Help_SecretKey);
@@ -164,6 +167,13 @@ void StaticPage::connectHelpButtons() {
     connect(mapper, SIGNAL(mapped(int)), this, SLOT(helpBtn_pressed(int)));
 }
 
+void StaticPage::loadSettings() {
+    QSettings settings;
+    ui->advStrongPw1Check->setChecked(settings.value(SG_STRONG_PW1).toBool());
+    ui->advStrongPw2Check->setChecked(settings.value(SG_STRONG_PW2).toBool());
+    ui->advStrongPw3Check->setChecked(settings.value(SG_STRONG_PW3).toBool());
+}
+
 void StaticPage::helpBtn_pressed(int helpIndex) {
     HelpBox help(this);
     help.setHelpIndex((HelpBox::Help)helpIndex);
@@ -209,7 +219,7 @@ void StaticPage::keyFound(bool found, bool* featuresMatrix) {
             if(!featuresMatrix[YubiKeyFinder::Feature_StaticPassword]) {
                 this->setEnabled(false);
             }
-        } else if(m_state >= State_Programming_Multiple) {
+        } else if(m_state >= State_Programming_Multiple && m_keyPresent == false) {
             if(this->currentIndex() == Page_Quick) {
                 if(m_state == State_Programming_Multiple) {
                     ui->quickWriteConfigBtn->setEnabled(true);
@@ -224,11 +234,14 @@ void StaticPage::keyFound(bool found, bool* featuresMatrix) {
                 }
             }
         }
+        m_keyPresent = true;
     } else {
         ui->quickBtn->setEnabled(true);
         ui->quickWriteConfigBtn->setEnabled(false);
         ui->advWriteConfigBtn->setEnabled(false);
 
+        m_keyPresent = false;
+
         if(m_state == State_Initial) {
             ui->quickConfigSlot2Radio->setEnabled(true);
             ui->advConfigSlot2Radio->setEnabled(true);
@@ -278,12 +291,13 @@ void StaticPage::resetQuickPage() {
     ui->quickAutoProgramKeysCheck->setChecked(false);
     ui->quickProgramMulKeysBox->setChecked(false);
 
-    ui->quickConfigProtectionCombo->setCurrentIndex(0);
+    ui->quickConfigProtectionBox->reset();
 
     ui->quickStaticLenTxt->setText("0");
     ui->quickStaticTxt->clear();
+    ui->quickScanCodesTxt->clear();
     ui->quickInsertTabBtn->setEnabled(true);
-    ui->quickClearBtn->setEnabled(false);
+    ui->quickClearBtn->setEnabled(true);
 
     ui->quickStopBtn->setEnabled(false);
     ui->quickResetBtn->setEnabled(false);
@@ -302,82 +316,48 @@ void StaticPage::freezeQuickPage(bool freeze) {
     ui->quickBackBtn->setEnabled(disable);
 }
 
-void StaticPage::on_quickConfigProtectionCombo_currentIndexChanged(int index) {
-    switch(index) {
-    case CONFIG_PROTECTION_DISABLED:
-        ui->quickCurrentAccessCodeTxt->clear();
-        ui->quickCurrentAccessCodeTxt->setEnabled(false);
-
-        ui->quickNewAccessCodeTxt->clear();
-        ui->quickNewAccessCodeTxt->setEnabled(false);
-        break;
-    case CONFIG_PROTECTION_ENABLE:
-        ui->quickCurrentAccessCodeTxt->clear();
-        ui->quickCurrentAccessCodeTxt->setEnabled(false);
-
-        on_quickNewAccessCodeTxt_editingFinished();
-        ui->quickNewAccessCodeTxt->setEnabled(true);
-        break;
-    case CONFIG_PROTECTION_DISABLE:
-    case CONFIG_PROTECTION_ENABLED:
-        on_quickCurrentAccessCodeTxt_editingFinished();
-        ui->quickCurrentAccessCodeTxt->setEnabled(true);
-
-        ui->quickNewAccessCodeTxt->clear();
-        ui->quickNewAccessCodeTxt->setEnabled(false);
-        break;
-    case CONFIG_PROTECTION_CHANGE:
-        on_quickCurrentAccessCodeTxt_editingFinished();
-        ui->quickCurrentAccessCodeTxt->setEnabled(true);
-
-        on_quickNewAccessCodeTxt_editingFinished();
-        ui->quickNewAccessCodeTxt->setEnabled(true);
-        break;
-    }
-}
-
-void StaticPage::on_quickCurrentAccessCodeTxt_editingFinished() {
-    QString txt = ui->quickCurrentAccessCodeTxt->text();
-    YubiKeyUtil::qstrClean(&txt, (size_t)ACC_CODE_SIZE * 2);
-    ui->quickCurrentAccessCodeTxt->setText(txt);
-}
-
-void StaticPage::on_quickNewAccessCodeTxt_editingFinished() {
-    QString txt = ui->quickNewAccessCodeTxt->text();
-    YubiKeyUtil::qstrClean(&txt, (size_t)ACC_CODE_SIZE * 2);
-    ui->quickNewAccessCodeTxt->setText(txt);
-}
 
 void StaticPage::on_quickHideParams_clicked(bool checked) {
     if(checked) {
         ui->quickStaticTxt->setEchoMode(QLineEdit::Password);
+        ui->quickScanCodesTxt->setEchoMode(QLineEdit::Password);
     } else {
         ui->quickStaticTxt->setEchoMode(QLineEdit::Normal);
+        ui->quickScanCodesTxt->setEchoMode(QLineEdit::Normal);
     }
 }
 
-void StaticPage::on_quickStaticTxt_textChanged(const QString &txt) {
-    int len = ui->quickStaticTxt->len();
+void StaticPage::on_quickStaticTxt_textEdited(const QString &txt) {
+    QString scanCodes = ScanEdit::textToScanCodes(txt);
+    ui->quickScanCodesTxt->setText(scanCodes);
 
+    int len = scanCodes.length() / 2;
     ui->quickStaticLenTxt->setText(QString::number(len));
     if(len >= MAX_SCAN_EDIT_SIZE) {
         ui->quickInsertTabBtn->setEnabled(false);
     } else {
         ui->quickInsertTabBtn->setEnabled(true);
     }
+}
 
-    ui->quickClearBtn->setEnabled(len > 0);
+void StaticPage::on_quickStaticTxt_returnPressed() {
+    QString text = ui->quickStaticTxt->text() + "\\n";
+    ui->quickStaticTxt->setText(text);
+    on_quickStaticTxt_textEdited(text);
 }
 
 void StaticPage::on_quickInsertTabBtn_clicked() {
-    ui->quickStaticTxt->injectScanCode(0x2b);
+    QString text = ui->quickStaticTxt->text() + "\\t";
+    ui->quickStaticTxt->setText(text);
     ui->quickStaticTxt->setFocus();
+    on_quickStaticTxt_textEdited(text);
 }
 
 void StaticPage::on_quickClearBtn_clicked() {
-    ui->quickStaticTxt->clearScanCodeText();
+    ui->quickScanCodesTxt->clear();
     ui->quickStaticTxt->clear();
-    ui->quickStaticTxt->setFocus();
+    ui->quickStaticLenTxt->setText("0");
+    ui->quickInsertTabBtn->setEnabled(true);
 }
 
 void StaticPage::on_quickWriteConfigBtn_clicked() {
@@ -419,7 +399,7 @@ void StaticPage::on_quickStopBtn_clicked() {
 }
 
 void StaticPage::stopQuickConfigWritting() {
-    qDebug() << "Stopping quick confgiuration writing...";
+    qDebug() << "Stopping quick configuration writing...";
 
     if(m_state >= State_Programming_Multiple) {
         ui->quickStopBtn->setEnabled(true);
@@ -469,23 +449,10 @@ bool StaticPage::validateQuickSettings() {
         }
     }
 
-    //Check if logging is disabled and
-    //configuration protection is being enabled
-    if(!settings.value(SG_ENABLE_CONF_PROTECTION).toBool() &&
-       !YubiKeyLogger::isLogging() &&
-       ui->quickConfigProtectionCombo->currentIndex() == CONFIG_PROTECTION_ENABLE) {
-        //Confirm from client
-        ConfirmBox confirm(this);
-        confirm.setConfirmIndex(ConfirmBox::Confirm_ConfigurationProtection);
-        int ret = confirm.exec();
-
-        switch (ret) {
-        case 1:     //Yes
-            break;
-        default:    //No
-            return false;
-        }
+    if(!ui->quickConfigProtectionBox->checkConfirm()) {
+        return false;
     }
+
     return true;
 }
 
@@ -506,6 +473,9 @@ void StaticPage::writeQuickConfig() {
     //Programming mode...
     m_ykConfig->setProgrammingMode(YubiKeyConfig::Mode_Static);
 
+    // set serial
+    m_ykConfig->setSerial(QString::number(YubiKeyFinder::getInstance()->serial()));
+
     //Configuration slot...
     int configSlot = 1;
     if(ui->quickConfigSlot2Radio->isChecked()) {
@@ -514,8 +484,10 @@ void StaticPage::writeQuickConfig() {
     m_ykConfig->setConfigSlot(configSlot);
 
     //Parameters...
-    QString staticTxt = ui->quickStaticTxt->scanCodeText();
+    QString staticTxt = ui->quickScanCodesTxt->text();
+
     YubiKeyUtil::qstrClean(&staticTxt, 0);
+    qDebug() << "static txt is: " << staticTxt;
 
     QString pubIdTxt("");
     QString pvtIdTxt("");
@@ -564,16 +536,11 @@ void StaticPage::writeQuickConfig() {
     }
 
     //Configuration protection...
-    //Current Access Code...
-    m_ykConfig->setCurrentAccessCodeTxt(ui->quickCurrentAccessCodeTxt->text());
-
-    //New Access Code...
-    if(ui->quickConfigProtectionCombo->currentIndex()
-        == CONFIG_PROTECTION_DISABLE){
-        m_ykConfig->setNewAccessCodeTxt(ACCESS_CODE_DEFAULT);
-    } else {
-        m_ykConfig->setNewAccessCodeTxt(ui->quickNewAccessCodeTxt->text());
-    }
+    m_ykConfig->setCurrentAccessCodeTxt(
+            ui->quickConfigProtectionBox->currentAccessCode());
+    m_ykConfig->setNewAccessCodeTxt(
+            ui->quickConfigProtectionBox->newAccessCode(),
+            ui->quickConfigProtectionBox->newAccMode());
 
     //Static Options...
     m_ykConfig->setShortTicket(true);
@@ -673,7 +640,7 @@ void StaticPage::resetAdvPage() {
     ui->advAutoProgramKeysCheck->setChecked(false);
     ui->advProgramMulKeysBox->setChecked(false);
 
-    ui->advConfigProtectionCombo->setCurrentIndex(0);
+    ui->advConfigProtectionBox->reset();
 
     ui->advStaticLen32Radio->setChecked(true);
     int minStaticLen = ui->advStaticLenBox->minimum();
@@ -687,10 +654,6 @@ void StaticPage::resetAdvPage() {
     ui->advSecretKeyTxt->clear();
     on_advSecretKeyTxt_editingFinished();
 
-    ui->advStrongPw1Check->setChecked(false);
-    ui->advStrongPw2Check->setChecked(false);
-    ui->advStrongPw3Check->setChecked(false);
-
     ui->advStopBtn->setEnabled(false);
     ui->advResetBtn->setEnabled(false);
 }
@@ -714,56 +677,10 @@ void StaticPage::on_advProgramMulKeysBox_clicked(bool checked) {
     }
 }
 
-void StaticPage::on_advConfigParamsCombo_currentIndexChanged(int index) {
+void StaticPage::on_advConfigParamsCombo_currentIndexChanged(__attribute__((unused)) int index) {
     changeAdvConfigParams();
 }
 
-void StaticPage::on_advConfigProtectionCombo_currentIndexChanged(int index) {
-    switch(index) {
-    case CONFIG_PROTECTION_DISABLED:
-        ui->advCurrentAccessCodeTxt->clear();
-        ui->advCurrentAccessCodeTxt->setEnabled(false);
-
-        ui->advNewAccessCodeTxt->clear();
-        ui->advNewAccessCodeTxt->setEnabled(false);
-        break;
-    case CONFIG_PROTECTION_ENABLE:
-        ui->advCurrentAccessCodeTxt->clear();
-        ui->advCurrentAccessCodeTxt->setEnabled(false);
-
-        on_advNewAccessCodeTxt_editingFinished();
-        ui->advNewAccessCodeTxt->setEnabled(true);
-        break;
-    case CONFIG_PROTECTION_DISABLE:
-    case CONFIG_PROTECTION_ENABLED:
-        on_advCurrentAccessCodeTxt_editingFinished();
-        ui->advCurrentAccessCodeTxt->setEnabled(true);
-
-        ui->advNewAccessCodeTxt->clear();
-        ui->advNewAccessCodeTxt->setEnabled(false);
-        break;
-    case CONFIG_PROTECTION_CHANGE:
-        on_advCurrentAccessCodeTxt_editingFinished();
-        ui->advCurrentAccessCodeTxt->setEnabled(true);
-
-        on_advNewAccessCodeTxt_editingFinished();
-        ui->advNewAccessCodeTxt->setEnabled(true);
-        break;
-    }
-}
-
-void StaticPage::on_advCurrentAccessCodeTxt_editingFinished() {
-    QString txt = ui->advCurrentAccessCodeTxt->text();
-    YubiKeyUtil::qstrClean(&txt, (size_t)ACC_CODE_SIZE * 2);
-    ui->advCurrentAccessCodeTxt->setText(txt);
-}
-
-void StaticPage::on_advNewAccessCodeTxt_editingFinished() {
-    QString txt = ui->advNewAccessCodeTxt->text();
-    YubiKeyUtil::qstrClean(&txt, (size_t)ACC_CODE_SIZE * 2);
-    ui->advNewAccessCodeTxt->setText(txt);
-}
-
 void StaticPage::on_advStaticLenBox_valueChanged(int value) {
     int len = value / 2 - KEY_SIZE;
     if(len > 0) {
@@ -785,7 +702,7 @@ void StaticPage::on_advStaticLen16Radio_clicked(bool checked) {
     enablePubId(!checked);
 }
 
-void StaticPage::on_advStaticLen32Radio_clicked(bool checked) {
+void StaticPage::on_advStaticLen32Radio_clicked(__attribute__((unused)) bool checked) {
     ui->advStaticLenBox->setEnabled(true);
     on_advStaticLenBox_valueChanged(ui->advStaticLenBox->value());
 }
@@ -839,13 +756,22 @@ void StaticPage::on_advSecretKeyGenerateBtn_clicked() {
             YubiKeyUtil::generateRandomHex((size_t)KEY_SIZE * 2));
 }
 
+void StaticPage::on_advStrongPw1Check_stateChanged(int state) {
+    QSettings settings;
+    settings.setValue(SG_STRONG_PW1, state != 0);
+}
+
 void StaticPage::on_advStrongPw2Check_stateChanged(int state) {
+    QSettings settings;
+    settings.setValue(SG_STRONG_PW2, state != 0);
     if(state == 0) {
         ui->advStrongPw3Check->setChecked(false);
     }
 }
 
 void StaticPage::on_advStrongPw3Check_stateChanged(int state) {
+    QSettings settings;
+    settings.setValue(SG_STRONG_PW3, state != 0);
     if(!ui->advStrongPw2Check->isChecked() && state > 0) {
         ui->advStrongPw2Check->setChecked(true);
     }
@@ -890,7 +816,7 @@ void StaticPage::on_advStopBtn_clicked() {
 }
 
 void StaticPage::stopAdvConfigWritting() {
-    qDebug() << "Stopping adv confgiuration writing...";
+    qDebug() << "Stopping adv configuration writing...";
 
     if(m_state >= State_Programming_Multiple) {
         ui->advStopBtn->setEnabled(true);
@@ -980,23 +906,10 @@ bool StaticPage::validateAdvSettings() {
         }
     }
 
-    //Check if logging is disabled and
-    //configuration protection is being enabled
-    if(!settings.value(SG_ENABLE_CONF_PROTECTION).toBool() &&
-       !YubiKeyLogger::isLogging() &&
-       ui->advConfigProtectionCombo->currentIndex() == CONFIG_PROTECTION_ENABLE) {
-        //Confirm from client
-        ConfirmBox confirm(this);
-        confirm.setConfirmIndex(ConfirmBox::Confirm_ConfigurationProtection);
-        int ret = confirm.exec();
-
-        switch (ret) {
-        case 1:     //Yes
-            break;
-        default:    //No
-            return false;
-        }
+    if(!ui->advConfigProtectionBox->checkConfirm()) {
+        return false;
     }
+
     return true;
 }
 
@@ -1017,6 +930,9 @@ void StaticPage::writeAdvConfig() {
     //Programming mode...
     m_ykConfig->setProgrammingMode(YubiKeyConfig::Mode_Static);
 
+    // set serial
+    m_ykConfig->setSerial(QString::number(YubiKeyFinder::getInstance()->serial()));
+
     //Configuration slot...
     int configSlot = 1;
     if(ui->advConfigSlot2Radio->isChecked()) {
@@ -1034,16 +950,11 @@ void StaticPage::writeAdvConfig() {
     m_ykConfig->setSecretKeyTxt(ui->advSecretKeyTxt->text());
 
     //Configuration protection...
-    //Current Access Code...
-    m_ykConfig->setCurrentAccessCodeTxt(ui->advCurrentAccessCodeTxt->text());
-
-    //New Access Code...
-    if(ui->advConfigProtectionCombo->currentIndex()
-        == CONFIG_PROTECTION_DISABLE){
-        m_ykConfig->setNewAccessCodeTxt(ACCESS_CODE_DEFAULT);
-    } else {
-        m_ykConfig->setNewAccessCodeTxt(ui->advNewAccessCodeTxt->text());
-    }
+    m_ykConfig->setCurrentAccessCodeTxt(
+            ui->advConfigProtectionBox->currentAccessCode());
+    m_ykConfig->setNewAccessCodeTxt(
+            ui->advConfigProtectionBox->newAccessCode(),
+            ui->advConfigProtectionBox->newAccMode());
 
     //Static Options...
     m_ykConfig->setStaticTicket(true);
@@ -1176,3 +1087,27 @@ void StaticPage::advUpdateResults(bool written, const QString &msg) {
     ui->advResultsWidget->resizeColumnsToContents();
     ui->advResultsWidget->resizeRowsToContents();
 }
+
+void StaticPage::on_quickScanCodesTxt_textEdited(const QString &scanCodes) {
+    QString text = ScanEdit::scanCodesToText(scanCodes);
+    ui->quickStaticTxt->setText(text);
+
+    int len = scanCodes.length() / 2;
+    ui->quickStaticLenTxt->setText(QString::number(len));
+
+    if(len >= MAX_SCAN_EDIT_SIZE) {
+        ui->quickInsertTabBtn->setEnabled(false);
+    } else {
+        ui->quickInsertTabBtn->setEnabled(true);
+    }
+}
+
+void StaticPage::setCurrentSlot(int slot) {
+    if(m_currentPage == Page_Advanced) {
+        ui->advConfigSlot1Radio->setChecked(slot == 1);
+        ui->advConfigSlot2Radio->setChecked(slot == 2);
+    } else if(m_currentPage == Page_Quick) {
+        ui->quickConfigSlot1Radio->setChecked(slot == 1);
+        ui->quickConfigSlot2Radio->setChecked(slot == 2);
+    }
+}
