@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2011-2012 Yubico AB.  All rights reserved.
+Copyright (C) 2011-2013 Yubico AB.  All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
@@ -32,13 +32,12 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <QObject>
 
 extern "C" {
-#include <ykcore.h>
-#include <ykdef.h>
 #include <ykpers.h>
-#include <yubikey.h>
 }
+
 #include "common.h"
-#include "qtimer.h"
+
+#define YK_VERSION(MAJOR, MINOR, BUILD) (MAJOR * 100 + MINOR * 10 + BUILD)
 
 class YubiKeyFinder : public QObject {
     Q_OBJECT
@@ -49,7 +48,7 @@ public:
 
     enum State {
         State_Absent,
-        State_Preset,
+        State_Present,
         State_Processing
     };
 
@@ -64,6 +63,10 @@ public:
         Feature_ChallengeResponse,
         Feature_SerialNumber,
         Feature_MovingFactor,
+        Feature_ChallengeResponseFixed,
+        Feature_Updatable,
+        Feature_Ndef,
+        Feature_LedInvert,
     };
 
     State state() const
@@ -75,6 +78,12 @@ public:
     unsigned int serial() const
     {return m_serial;}
 
+    int touchLevel() const
+    {return m_touchLevel;}
+
+    YK_STATUS *status() const
+    { return m_ykds;}
+
     QString versionStr();
     void reportError();
 
@@ -88,15 +97,19 @@ private:
 
     static YubiKeyFinder* _instance;
     static const unsigned int FEATURE_MATRIX[][2];
+    static const unsigned int FEATURE_MATRIX_EXCLUDE[][2];
 
     State m_state;
     YK_KEY *m_yk;
+    YK_STATUS *m_ykds;
     unsigned int m_version;
     unsigned int m_versionMinor;
     unsigned int m_versionMajor;
     unsigned int m_versionBuild;
     unsigned int m_serial;
 
+    int m_touchLevel;
+
     QTimer *m_timer;
 
 public slots:
